from selenium.webdriver.common.keys import Keys

from .framework import (
    selenium_test,
    SeleniumTestCase
)


class CollectionEditTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_change_dbkey_simple_list(self):
        self.use_beta_history()
        self.create_simple_list_collection()
        self.open_collection_edit_view()
        self.navigate_to_database_tab()
        dbkeyValue = "unspecified"
        self.check_current_dbkey_value(dbkeyValue)
        dbkeyNew = "hg17"
        self.change_dbkey_value_and_click_submit(dbkeyValue, dbkeyNew)
        self.history_panel_wait_for_hid_ok(4)
        self.use_beta_history()
        self.open_collection_edit_view()
        self.navigate_to_database_tab()
        self.check_current_dbkey_value(dbkeyNew)

    def create_simple_list_collection(self):
        self.perform_upload(self.get_filename("1.fasta"))
        self._wait_for_and_select([1])

        self._collection_dropdown("build list")

        self.collection_builder_set_name("my cool list")
        self.screenshot("collection_builder_list")
        self.collection_builder_create()
        self._wait_for_hid_visible(2)

    def open_collection_edit_view(self):
        self.components.history_panel.collection_menu_edit_attributes.wait_for_and_click()

    def navigate_to_database_tab(self):
        self.components.edit_collection_attributes.database_genome_tab.wait_for_and_click()

    def check_current_dbkey_value(self, dbkeyValue):
        self.components.edit_collection_attributes.database_value(dbkey=dbkeyValue).wait_for_visible()

    def change_dbkey_value_and_click_submit(self, dbkeyValue, dbkeyNew):
        self.components.edit_collection_attributes.database_value(dbkey=dbkeyValue).wait_for_and_click()
        self.driver.find_element_by_css_selector("input.multiselect__input").send_keys(dbkeyNew)
        self.driver.find_element_by_css_selector("input.multiselect__input").send_keys(Keys.ENTER)
        self.components.edit_collection_attributes.save_btn.wait_for_and_click()

    def _wait_for_and_select(self, hids):
        """
        Waits for uploads to pass through queued, running, ok. Not all the states are not guaranteed
        depending on how fast the upload goes compared to the history polling updates, it might just
        skip to the end for a really fast upload
        """

        for hid in hids:
            timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
            row_selector = self.content_item_by_attributes(hid=hid, state="ok")
            row = self.wait_for_present(row_selector, timeout=timeout)
            row.send_keys(" ")

    def _collection_dropdown(self, option_description):
        return self.use_bootstrap_dropdown(option=option_description, menu="selected content menu")

    def _wait_for_hid_visible(self, hid, state="ok"):
        timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
        row_selector = self.content_item_by_attributes(hid=hid, state=state)
        self.wait_for_visible(row_selector, timeout=timeout)



from .framework import (
    selenium_test,
    SeleniumTestCase
)

# Remove hack when submit_login works more consistently.
VALID_LOGIN_RETRIES = 3


class HistorySharingTestCase(SeleniumTestCase):

    @selenium_test
    def test_sharing_valid(self):
        user1_email, user2_email, history_id = self.setup_two_users_with_one_shared_history()
        self.submit_login(user2_email, retries=VALID_LOGIN_RETRIES)
        response = self.api_get(f"histories/{history_id}", raw=True)
        assert response.status_code == 200, response.text

    @selenium_test
    def test_sharing_valid_by_id(self):
        user1_email, user2_email, history_id = self.setup_two_users_with_one_shared_history(share_by_id=True)
        self.submit_login(user2_email, retries=VALID_LOGIN_RETRIES)
        response = self.api_get(f"histories/{history_id}", raw=True)
        assert response.status_code == 200, response.text

    @selenium_test
    def test_unsharing(self):
        user1_email, user2_email, history_id = self.setup_two_users_with_one_shared_history()
        self.submit_login(user1_email, retries=VALID_LOGIN_RETRIES)
        self.navigate_to_history_share_page()
        self.components.histories.sharing.share_with_collapse.wait_for_and_click()

        unshare_user_button = self.components.histories.sharing.unshare_user_button
        unshare_user_button.wait_for_and_click()
        self.components.histories.sharing.submit_sharing_with.wait_for_and_click()

        self.navigate_to_history_share_page()
        unshare_user_button.assert_absent()

        self.logout_if_needed()
        self.submit_login(user2_email, retries=VALID_LOGIN_RETRIES)
        response = self.api_get(f"histories/{history_id}", raw=True)
        assert response.status_code == 403

    @selenium_test
    def test_unshared_history_inaccessible(self):
        # Here for completeness for now - but probably should have an explict API test case.
        user1_email = self._get_random_email()
        user2_email = self._get_random_email()

        self.register(user1_email)
        history_id = self.current_history_id()
        self.logout_if_needed()

        self.register(user2_email)
        response = self.api_get(f"histories/{history_id}", raw=True)
        assert response.status_code == 403

    @selenium_test
    def test_sharing_with_invalid_user(self):
        user1_email = self._get_random_email()
        self.register(user1_email)
        self.share_history_with_user(user_email="invalid_user@test.com")
        self.assert_error_message(contains='is not a valid Galaxy user')

    @selenium_test
    def test_sharing_with_self(self):
        user1_email = self._get_random_email()
        self.register(user1_email)
        self.share_history_with_user(user_email=user1_email)
        self.assert_error_message(contains='You cannot share resources with yourself')

    def setup_two_users_with_one_shared_history(self, share_by_id=False):
        user1_email = self._get_random_email()
        user2_email = self._get_random_email()

        self.register(user1_email)
        self.logout_if_needed()
        self.register(user2_email)
        user2_id = self.api_get("users")[0]["id"]
        self.logout_if_needed()

        self.submit_login(user1_email, retries=VALID_LOGIN_RETRIES)
        # Can't share an empty history...
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()

        history_id = self.current_history_id()

        if share_by_id:
            self.share_history_with_user(user_email=user2_email, assert_valid=True)
        else:
            self.share_history_with_user(user_id=user2_id, user_email=user2_email, assert_valid=True)
        self.logout_if_needed()

        return user1_email, user2_email, history_id

    def navigate_to_history_share_page(self):
        self.home()
        self.click_history_option("Share or Publish")

    def share_history_with_user(self, user_id=None, user_email=None, assert_valid=False, screenshot=False):
        """Share the current history with a target user by ID or email.

        ``user_email`` will be used to enter in the share form unless ``user_id``
        is also specified. The ``user_email`` however is always used to check
        the result if ``assert_valid`` is True.
        """
        self.navigate_to_history_share_page()
        self.components.histories.sharing.share_with_collapse.wait_for_and_click()
        multiselect = self.components.histories.sharing.share_with_multiselect.wait_for_and_click()
        self.components.histories.sharing.share_with_input.wait_for_and_send_keys(user_id or user_email)
        self.send_enter(multiselect)

        if screenshot:
            self.screenshot("history_sharing_user")

        self.components.histories.sharing.submit_sharing_with.wait_for_and_click()

        if assert_valid:
            self.assert_no_error_message()

            xpath = f'//span[contains(text(), "{user_email}")]'
            self.wait_for_xpath_visible(xpath)



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class SizzleLoadingTestCase(SeleniumTestCase):

    @selenium_test
    def test_sizzle_loads(self):
        self.home()
        self.wait_for_sizzle_selector_clickable("div")



import pytest

from galaxy.selenium.navigates_galaxy import (
    edit_details
)
from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase,
)


NEW_HISTORY_NAME = "New History Name"


class HistoryPanelTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_history_panel_landing_state(self):
        self.assert_initial_history_panel_state_correct()
        name_element = self.history_panel_name_element()
        if self.is_beta_history():
            # look for the editor icon
            editor = self.components.history_panel.editor.selector(scope=".beta.history")
            toggle = editor.toggle
            toggle.wait_for_visible()
        else:
            tag_icon_selector = self.navigation.history_panel.selectors.tag_icon
            annotation_icon_selector = self.navigation.history_panel.selectors.annotation_icon
            self.wait_for_visible(tag_icon_selector)
            self.wait_for_visible(annotation_icon_selector)
            self.assert_tooltip_text(name_element, self.navigation.history_panel.text.tooltip_name)

    @selenium_test
    def test_history_panel_rename(self):
        self.history_panel_rename(NEW_HISTORY_NAME)
        self.assert_name_changed()

    @selenium_test
    def test_history_rename_confirm_with_click(self):
        if self.is_beta_history():
            raise pytest.skip("Beta History Panel has explicit editing toggle mode, so can not click off to the side to save")
        editable_text_input_element = self.history_panel_name_input()
        editable_text_input_element.send_keys(NEW_HISTORY_NAME)
        self.click_center()
        self.assert_absent(self.navigation.history_panel.selectors.name_edit_input)
        self.assert_name_changed()

    @selenium_test
    def test_history_rename_cancel_with_escape(self):
        self.open_history_editor()
        editable_text_input_element = self.history_panel_name_input()
        editable_text_input_element.send_keys(NEW_HISTORY_NAME)
        self.send_escape(editable_text_input_element)
        self.components.history_panel.name_edit_input.wait_for_absent_or_hidden()
        assert NEW_HISTORY_NAME not in self.history_panel_name()

    @selenium_test
    @edit_details
    def test_history_tags_and_annotations_buttons(self):
        if self.is_beta_history():
            history_editor = self.components.history_panel.editor.selector(scope=".beta.history")
            history_editor.annotation_input.wait_for_clickable()
            history_editor.tags_input.wait_for_clickable()
        else:
            tag_icon_selector = self.navigation.history_panel.selectors.tag_icon
            annotation_icon_selector = self.navigation.history_panel.selectors.annotation_icon

            tag_area_selector = self.navigation.history_panel.selectors.tag_area
            annotation_area_selector = self.navigation.history_panel.selectors.annotation_area

            tag_icon = self.wait_for_clickable(tag_icon_selector)
            annon_icon = self.wait_for_clickable(annotation_icon_selector)

            self.assert_absent_or_hidden(tag_area_selector)
            self.assert_absent_or_hidden(annotation_area_selector)

            tag_icon.click()

            self.wait_for_visible(tag_area_selector)
            self.assert_absent_or_hidden(annotation_area_selector)

            tag_icon.click()
            self.sleep_for(self.wait_types.UX_TRANSITION)
            annon_icon.click()

            self.wait_for_visible(annotation_area_selector)
            self.assert_absent_or_hidden(tag_area_selector)

            annon_icon.click()
            self.sleep_for(self.wait_types.UX_TRANSITION)

            self.assert_absent_or_hidden(tag_area_selector)
            self.assert_absent_or_hidden(annotation_area_selector)

    @selenium_test
    def test_history_panel_annotations_change(self):
        history_panel = self.components.history_panel

        @retry_assertion_during_transitions
        def assert_current_annotation(expected, error_message="History annotation",
                                      is_equal=True):

            text_component = history_panel.annotation_editable_text
            current_annotation = text_component.wait_for_visible()
            error_message += " given: [%s] expected [%s] "
            if is_equal:
                assert current_annotation.text == expected, error_message % (
                    current_annotation.text, expected)
            else:
                assert current_annotation.text != expected, error_message % (
                    current_annotation.text, expected)

        def set_random_annotation(clear_text=True):
            random_annotation = self._get_random_name(prefix="arbitrary_annotation_")
            self.set_history_annotation(random_annotation, clear_text)
            return random_annotation

        # assert that annotation wasn't set before
        history_panel.annotation_area.assert_absent_or_hidden()

        # assign annotation random text
        initial_annotation = set_random_annotation()
        assert_current_annotation(initial_annotation)

        # change annotation text
        changed_annotation = set_random_annotation()

        assert_current_annotation(initial_annotation, error_message="History annotation was not changed!",
                                  is_equal=False)
        assert_current_annotation(changed_annotation,
                                  error_message="History annotation was changed, but annotation text is wrong!",
                                  is_equal=True)

    @selenium_test
    def test_history_panel_tags_change(self):

        def create_tags(size):
            history_panel_tags = list()
            for i in range(size):
                history_panel_tags.append(self._get_random_name(prefix="arbitrary_tag_%s_") % i)
            return history_panel_tags

        def add_tags(tags_size):
            tags = create_tags(tags_size)
            self.history_panel_add_tags(tags)
            return tags

        # check tags against list
        def assert_current_tags(expected_tags):
            if self.is_beta_history():
                current_tags = self.open_tags()
            else:
                current_tags = self.components.history_panel.tags
            errmsg = f"tags [{current_tags}] are not the same as expected [{expected_tags}]"
            assert [tag.text for tag in current_tags.all()] == expected_tags, errmsg

        # looks like this is intended to check if the tag editor is open
        def assert_no_tags():
            if self.is_beta_history():
                tags_component = self.components.history_panel.tag_editor.selector(scope=".beta.history")
                tags_component.display.assert_absent_or_hidden()
            else:
                self.components.history_panel.tag_area.assert_absent_or_hidden()

        assert_no_tags()

        # add new tags to empty tags area
        tags_size = 5
        tags = add_tags(tags_size)
        assert_current_tags(tags)

        # add more tags to non-empty tags area
        tags += add_tags(tags_size)
        assert_current_tags(tags)

        # delete all tags
        expected_tags_len = len(tags)
        self.clear_tags(expected_tags_len)

        assert_no_tags()

    # after about 5 tags, a toggle link shows up and you have to click it to see the full list
    def open_tags(self):
        tags_component = self.components.history_panel.tag_editor.selector(scope=".beta.history")
        if not tags_component.toggle.is_absent:
            tags_component.toggle.wait_for_and_click()
        tags_component.display.wait_for_visible()
        return tags_component.display

    @edit_details
    def clear_tags(self, expected_tags_size):
        if self.is_beta_history():
            self.open_tags()
            tags = self.components.history_panel.tag_editor.selector(scope=".beta.history")
            close_tag_buttons = tags.tag_close_btn.all()
        else:
            close_tag_buttons = self.components.history_panel.tag_close_btn.all()

        current_tags_size = len(close_tag_buttons)

        errmsg = f"there are more tags than expected! current {current_tags_size}, expected {expected_tags_size}"
        assert expected_tags_size == current_tags_size, errmsg

        for close_btn in reversed(close_tag_buttons):
            close_btn.click()
            self.sleep_for(self.wait_types.UX_RENDER)

        if not self.is_beta_history():
            self.components.history_panel.tag_icon.wait_for_and_click()
            self.sleep_for(self.wait_types.UX_RENDER)

    @selenium_test
    def test_refresh_preserves_state(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()

        # Open the details, verify they are open and do a refresh.
        self.history_panel_ensure_showing_item_details(hid=1)
        self.history_panel_item_body_component(1, wait=True)

        self._refresh()
        self.wait_for_history()

        # After the refresh, verify the details are still open.
        self.sleep_for(self.wait_types.UX_TRANSITION)
        self.wait_for_selector_clickable(self.history_panel_item_selector(hid=1))
        assert self.history_panel_item_showing_details(hid=1)

        # Close the detailed display, refresh, and ensure they are still closed.
        wait = not self.is_beta_history()
        self.history_panel_click_item_title(hid=1, wait=wait)
        assert not self.history_panel_item_showing_details(hid=1)

        self._refresh()

        self.sleep_for(self.wait_types.UX_TRANSITION)
        self.wait_for_selector_clickable(self.history_panel_item_selector(hid=1))
        assert not self.history_panel_item_showing_details(hid=1)

    @retry_assertion_during_transitions
    def assert_name_changed(self):
        name = self.history_panel_name()
        self.assertEqual(name, NEW_HISTORY_NAME)

    def _refresh(self):
        if self.is_beta_history():
            # beta history has no refresh button
            self.home()
        else:
            self.history_panel_refresh_click()



"""Variant of JuypterContextImpl that can also use populators.

This provides an environment separate from test cases that can leaverage
both Selenium for testing Galaxy with a browser and API populators for filling
in fixture data rapidly in the target Galaxy.
"""
from typing import Optional

from galaxy.selenium.context import init as base_init
from galaxy.selenium.jupyter_context import JupyterContextImpl
from galaxy_test.base.api_util import get_admin_api_key
from .framework import GalaxyTestSeleniumContext


class JupyterTestContextImpl(JupyterContextImpl, GalaxyTestSeleniumContext):
    # Reload components interactively to limit number of Python kernel
    # restarts needed during test building.
    _interactive_components = True

    def __init__(self, from_dict: Optional[dict] = None) -> None:
        from_dict = from_dict or {}
        super().__init__(from_dict)
        self.admin_api_key = from_dict.get("admin_api_key", get_admin_api_key())
        self.login_email = from_dict.get("login_email")
        self.login_password = from_dict.get("login_password")

    def test_login(self):
        self.home()
        self.submit_login(self.login_email, self.login_password)


def init(config=None):
    return base_init(config, clazz=JupyterTestContextImpl)



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class RegistrationTestCase(SeleniumTestCase):

    @selenium_test
    def test_landing(self):
        # loading galaxy homepage
        self.home()
        assert self.driver.title == "Galaxy", self.driver.title
        self.components.masthead._.wait_for_visible()

    @selenium_test
    def test_registration(self):
        self.home()
        self.register()

    @selenium_test
    def test_logout(self):
        self.home()
        self.register()
        assert self.is_logged_in()
        self.logout_if_needed()
        assert not self.is_logged_in()
        self.home()
        self.components.masthead.username.assert_absent_or_hidden()

    @selenium_test
    def test_reregister_email_fails(self):
        self.home()
        email = self._get_random_email()
        password = self.default_password
        confirm = password
        username = email.split("@")[0]
        self.register(email, password, username, confirm)
        self.logout_if_needed()
        self.register(email, password, username, confirm, assert_valid=False)
        self.assert_error_message()

    @selenium_test
    def test_reregister_username_fails(self):
        self.home()
        email1 = self._get_random_email()
        email2 = self._get_random_email()
        password = self.default_password
        confirm = password
        username = email1.split("@")[0]
        self.register(email1, password, username, confirm)
        self.logout_if_needed()
        self.register(email2, password, username, confirm, assert_valid=False)
        self.assert_error_message(contains='Public name is taken')

    @selenium_test
    def test_bad_emails(self):
        bad_emails = ['bob', 'bob@', 'bob@idontwanttocleanup', 'bob.cantmakeme']
        good_email = self._get_random_email()
        password = self.default_password
        confirm = password
        username = good_email.split("@")[0]
        for bad_email in bad_emails:
            self.register(bad_email, password, username, confirm, assert_valid=False)
            self.assert_error_message(contains='The format of the email address is not correct.')

    @selenium_test
    def test_short_password(self):
        self.register(password="1234", assert_valid=False)
        self.assert_error_message(contains='Use a password of at least 6 characters')

    @selenium_test
    def test_password_confirmation(self):
        bad_confirms = ['1234', '12345678', '123456 7']
        for bad_confirm in bad_confirms:
            self.register(confirm=bad_confirm, assert_valid=False)
            self.assert_error_message(contains='Passwords do not match')

    @selenium_test
    def test_bad_usernames(self):
        bad_usernames = ['BOBERT', 'Robert Paulson', 'bobert!']
        for bad_username in bad_usernames:
            self.register(username=bad_username, assert_valid=False)
            self.assert_error_message(contains='Public name must contain only ')



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class CollectionBuildersTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_build_list_simple_hidden(self):
        self.perform_upload(self.get_filename("1.fasta"))

        self._wait_for_and_select([1])

        if self.is_beta_history():
            self._collection_dropdown("build list")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_list)

        self.collection_builder_set_name("my cool list")
        self.screenshot("collection_builder_list")
        self.collection_builder_create()
        self._wait_for_hid_visible(2)

    @selenium_test
    def test_build_list_and_show_items(self):
        self.perform_upload(self.get_filename("1.fasta"))

        self._wait_for_and_select([1])

        if self.is_beta_history():
            self._collection_dropdown("build list")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_list)

        # this toggles the checkbox to not hide originals
        self.collection_builder_hide_originals()
        self.collection_builder_set_name("my cool list")
        self.collection_builder_create()
        self.home()  # this shouldn't be necessary, and it isn't with a real browser.
        self._wait_for_hid_visible(3)

    @selenium_test
    def test_build_pair_simple_hidden(self):
        self.perform_upload(self.get_filename("1.tabular"))
        self.perform_upload(self.get_filename("2.tabular"))

        self._wait_for_and_select([1, 2])

        if self.is_beta_history():
            self._collection_dropdown("build pair")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_pair)

        self.collection_builder_set_name("my awesome pair")
        self.screenshot("collection_builder_pair")
        self.collection_builder_create()
        self._wait_for_hid_visible(3)

    @selenium_test
    def test_build_paired_list_simple(self):
        self.perform_upload(self.get_filename("1.tabular"))
        self.perform_upload(self.get_filename("2.tabular"))

        # self.home()
        self._wait_for_and_select([1, 2])

        if self.is_beta_history():
            self._collection_dropdown("build list of pairs")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_list_pairs)

        self.collection_builder_clear_filters()
        self.collection_builder_click_paired_item("forward", 0)
        self.collection_builder_click_paired_item("reverse", 1)
        self.collection_builder_set_name("my awesome paired list")
        self.screenshot("collection_builder_paired_list")
        self.collection_builder_create()

        if self.is_beta_history():
            self._wait_for_hid_visible(3)
            # switch to hidden filters to see the hidden datasets appear
            self._show_hidden_content()
            self._wait_for_hid_visible(1)
            self._wait_for_hid_visible(2)
        else:
            self.history_panel_wait_for_hid_ok(3)
            self.history_panel_wait_for_hid_hidden(1)
            self.history_panel_wait_for_hid_hidden(2)

    @selenium_test
    def test_build_paired_list_show_original(self):
        self.perform_upload(self.get_filename("1.tabular"))
        self.perform_upload(self.get_filename("2.tabular"))

        # self.home()  # should not be necessary, chromedriver fails without it though
        self._wait_for_and_select([1, 2])

        if self.is_beta_history():
            self._collection_dropdown("build list of pairs")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_list_pairs)

        self.wait_for_and_click(self.navigation.collection_builders.selectors.clear_filters)

        forward_column = self.wait_for_visible(self.navigation.collection_builders.selectors.forward_datasets)
        first_datset_forward = forward_column.find_elements_by_css_selector("li")[0]
        first_datset_forward.click()

        reverse_column = self.wait_for_visible(self.navigation.collection_builders.selectors.reverse_datasets)
        second_dataset_reverse = reverse_column.find_elements_by_css_selector("li")[1]
        second_dataset_reverse.click()

        self.collection_builder_hide_originals()
        self.collection_builder_set_name("my awesome paired list")
        self.collection_builder_create()

        if self.is_beta_history():
            self._wait_for_hid_visible(5)
            self._wait_for_hid_visible(1)
            self._wait_for_hid_visible(2)
            self._show_hidden_content()
            self._wait_for_hid_visible(3)
            self._wait_for_hid_visible(4)
        else:
            self.history_panel_wait_for_hid_ok(5)
            self.history_panel_refresh_click()
            self.history_panel_wait_for_hid_ok(1)
            self.history_panel_wait_for_hid_ok(2)
            self.history_panel_wait_for_hid_hidden(3)
            self.history_panel_wait_for_hid_hidden(4)

    @selenium_test
    def test_build_simple_list_via_rules_hidden(self):
        self.perform_upload(self.get_filename("1.fasta"))

        self._wait_for_and_select([1])

        if self.is_beta_history():
            self._collection_dropdown("build collection from rules")
        else:
            self.history_panel_multi_operation_action_click(self.navigation.history_panel.multi_operations.labels.build_from_rules)

        self.collection_builder_set_name("my cool list")
        self.screenshot("collection_builder_rules_list")
        self.collection_builder_create()

        self._wait_for_hid_visible(2)

        if self.is_beta_history():
            self._show_hidden_content()
            self._wait_for_hid_visible(1)
        else:
            self.history_panel_wait_for_hid_hidden(1)

    def _wait_for_hid_visible(self, hid, state="ok"):
        if self.is_beta_history():
            # takes a little while for these things to upload and end up in the history
            timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
            row_selector = self.content_item_by_attributes(hid=hid, state=state)
            self.wait_for_visible(row_selector, timeout=timeout)
        else:
            self.history_panel_wait_for_hid_visible(hid, allowed_force_refreshes=1)

    def _collection_dropdown(self, option_description):
        return self.use_bootstrap_dropdown(option=option_description, menu="selected content menu")

    def _wait_for_and_select(self, hids):
        """
        Waits for uploads to pass through queued, running, ok. Not all the states are not guaranteed
        depending on how fast the upload goes compared to the history polling updates, it might just
        skip to the end for a really fast upload
        """

        if self.is_beta_history():
            for hid in hids:
                timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
                row_selector = self.content_item_by_attributes(hid=hid, state="ok")
                row = self.wait_for_present(row_selector, timeout=timeout)
                row.send_keys(" ")
        else:
            for hid in hids:
                self.history_panel_wait_for_hid_ok(hid)
            self.history_panel_multi_operations_show()
            for hid in hids:
                self.history_panel_muli_operation_select_hid(hid)

    def _show_hidden_content(self):
        """Switches the beta hidden filter toggle on """
        self.sleep_for(self.wait_types.UX_RENDER)
        self.beta_history_element("content filter toggle").wait_for_and_click()
        self.beta_history_element("show hidden filter toggle").wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)



from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase,
)


class LibraryLandingTestCase(SeleniumTestCase):

    requires_admin = True

    def setup_with_driver(self):
        super().setup_with_driver()
        self.admin_login()
        self.libraries_open()

    @selenium_test
    def test_create_new_close(self):
        num_displayed_libraries = self._num_displayed_libraries()
        self.components.libraries.create_new_library_btn.wait_for_and_click()
        self.wait_for_selector_visible(".new-row")
        self.screenshot("libraries_new")
        self.components.libraries.create_new_library_btn.wait_for_and_click()
        self.wait_for_overlays_cleared()
        assert self._num_displayed_libraries() == num_displayed_libraries

    @selenium_test
    def test_create_new(self):
        name = self._get_random_name(prefix="testcreatenew")
        self.libraries_index_create(name)

        # Must be showing at least one library now.
        self._assert_at_least_one_library_displayed()

        self._search_for_only_with_name(name)

    @selenium_test
    def test_rename(self):
        name = self._get_random_name(prefix="testprerename")
        self.libraries_index_create(name)
        self._search_for_only_with_name(name)

        edit_button = self.wait_for_selector_clickable(".edit_library_btn")
        edit_button.click()

        new_name = self._get_random_name(prefix="testpostrename")
        name_box = self.wait_for_selector_clickable(".input_library_name")
        name_box.send_keys(new_name)

        self.screenshot("libraries_rename")
        save_button = self.wait_for_selector_clickable(".save_library_btn")
        save_button.click()

        self._search_for_only_with_name(new_name)

    @selenium_test
    def test_sorting(self):
        # Throw in another library to ensure filtering is working...
        other = self._get_random_name(prefix="notthesame")
        self.libraries_index_create(other)
        self.wait_for_overlays_cleared()

        namebase = self._get_random_name(prefix="testsort")
        self.libraries_index_create(f"{namebase} b")
        self.wait_for_overlays_cleared()
        self.libraries_index_create(f"{namebase} a")
        self.wait_for_overlays_cleared()
        self.libraries_index_create(f"{namebase} c")

        self.screenshot("libraries_index")

        self.libraries_index_search_for(namebase)

        self._assert_num_displayed_libraries_is(3)
        self.screenshot("libraries_index_search")
        # sort ascending
        self.libraries_index_sort_click()
        self._assert_names_are([f"{namebase} a", f"{namebase} b", f"{namebase} c"])
        # sort descending
        self.libraries_index_sort_click()
        self._assert_names_are([f"{namebase} c", f"{namebase} b", f"{namebase} a"])

    def _search_for_only_with_name(self, name):
        self.libraries_index_search_for(name)
        self._assert_num_displayed_libraries_is(1)

    @retry_assertion_during_transitions
    def _assert_names_are(self, expected_names):
        names = [e.find_element_by_css_selector("td a").text for e in self.libraries_index_table_elements()]
        self.assertEqual(names, expected_names)

    @retry_assertion_during_transitions
    def _assert_at_least_one_library_displayed(self):
        assert self._num_displayed_libraries() > 0

    @retry_assertion_during_transitions
    def _assert_num_displayed_libraries_is(self, n):
        self.assertEqual(n, self._num_displayed_libraries())

    def _num_displayed_libraries(self):
        return len(self.libraries_index_table_elements())



from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SharedStateSeleniumTestCase,
)


class CustomBuildsTestcase(SharedStateSeleniumTestCase):

    @selenium_test
    def test_build_add(self):
        self._login()

        self.navigate_to_custom_builds_page()

        self.add_custom_build(self.build_name1, self.build_key1)
        self.assert_custom_builds_in_grid([self.build_name1])

    @selenium_test
    def test_build_delete(self):
        self._login()

        self.navigate_to_custom_builds_page()

        self.add_custom_build(self.build_name2, self.build_key2)
        self.assert_custom_builds_in_grid([self.build_name2])

        self.delete_custom_build(self.build_name2)
        self.assert_custom_builds_in_grid([self.build_name2], False)

    @retry_assertion_during_transitions
    def assert_custom_builds_in_grid(self, expected_builds, present=True):
        actual_builds = self.get_custom_builds()
        intersection = set(actual_builds).intersection(expected_builds)
        if present:
            self.assertEqual(intersection, set(expected_builds))
        else:
            self.assertEqual(intersection, set())

    def _login(self):
        self.home()  # ensure Galaxy is loaded
        self.submit_login(self.user_email, retries=2)

    def add_custom_build(self, build_name, build_key):
        name_input = self.wait_for_selector('input[id="name"]')
        name_input.send_keys(build_name)

        key_input = self.wait_for_selector('input[id="id"]')
        key_input.send_keys(build_key)

        len_type_select = self.wait_for_selector('select[id="type"]')
        len_type_select.click()

        option = self.wait_for_sizzle_selector_clickable('option[value="text"]')
        option.click()
        content_area = self.wait_for_and_click_selector('textarea[id="len-file-text-area"]')
        content_area.send_keys('content')

        self.wait_for_and_click_selector('button#save')

    def delete_custom_build(self, build_name):
        delete_button = None
        grid = self.wait_for_selector('table.grid > tbody')
        for row in grid.find_elements_by_tag_name('tr'):
            td = row.find_elements_by_tag_name('td')
            name = td[0].text
            if name == build_name:
                delete_button = td[3].find_element_by_css_selector('.fa-trash-o')
                break

        if delete_button is None:
            raise AssertionError(f'Failed to find custom build with name [{build_name}]')

        delete_button.click()

    def get_custom_builds(self):
        self.sleep_for(self.wait_types.UX_RENDER)
        builds = []
        grid = self.wait_for_selector('table.grid > tbody')
        for row in grid.find_elements_by_tag_name('tr'):
            name = row.find_elements_by_tag_name('td')[0].text
            builds.append(name)
        return builds

    def navigate_to_custom_builds_page(self):
        self.home()
        self.click_masthead_user()  # Open masthead menu
        self.components.masthead.custom_builds.wait_for_and_click()

    def setup_shared_state(self):
        CustomBuildsTestcase.user_email = self._get_random_email()
        self.register(self.user_email)

        CustomBuildsTestcase.build_name1 = self._get_random_name()
        CustomBuildsTestcase.build_name2 = self._get_random_name()
        CustomBuildsTestcase.build_key1 = self._get_random_name(len=5)
        CustomBuildsTestcase.build_key2 = self._get_random_name(len=5)



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class HistoryOptionsTestCase(SeleniumTestCase):

    @selenium_test
    def test_options(self):
        self.register()
        self.perform_upload(self.get_filename("1.txt"))
        self.click_history_options()

        menu_selector = self.navigation.history_panel.selectors.options_menu
        self.wait_for_visible(menu_selector)

        # Click away closes history options
        self.click_center()

        self.wait_for_absent_or_hidden(menu_selector)

        hid = 1
        self.history_panel_wait_for_hid_state(hid, 'ok')
        self.history_panel_click_item_title(hid=hid, wait=True)
        item_component = self.history_panel_item_body_component(hid=hid)
        item_component.wait_for_visible()
        self.history_panel_click_item_title(hid=hid, wait=True)
        item_component.assert_absent_or_hidden_after_transitions()



import yaml

from galaxy_test.base import rules_test_data
from galaxy_test.base.populators import load_data_dict
from galaxy_test.base.workflow_fixtures import (
    WORKFLOW_NESTED_REPLACEMENT_PARAMETER,
    WORKFLOW_NESTED_RUNTIME_PARAMETER,
    WORKFLOW_NESTED_SIMPLE,
    WORKFLOW_RENAME_ON_REPLACEMENT_PARAM,
    WORKFLOW_RUNTIME_PARAMETER_SIMPLE,
    WORKFLOW_SELECT_FROM_OPTIONAL_DATASET,
    WORKFLOW_SIMPLE_CAT_TWICE,
    WORKFLOW_WITH_CUSTOM_REPORT_1,
    WORKFLOW_WITH_CUSTOM_REPORT_1_TEST_DATA,
    WORKFLOW_WITH_DYNAMIC_OUTPUT_COLLECTION,
    WORKFLOW_WITH_OLD_TOOL_VERSION,
    WORKFLOW_WITH_RULES_1,
)
from .framework import (
    managed_history,
    selenium_test,
    SeleniumTestCase,
    UsesHistoryItemAssertions,
)


class WorkflowRunTestCase(SeleniumTestCase, UsesHistoryItemAssertions):

    ensure_registered = True

    @selenium_test
    @managed_history
    def test_simple_execution(self):
        self.perform_upload(self.get_filename("1.fasta"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_SIMPLE_CAT_TWICE)
        self.screenshot("workflow_run_simple_ready")
        self.workflow_run_submit()
        self.sleep_for(self.wait_types.UX_TRANSITION)
        self.screenshot("workflow_run_simple_submitted")
        if self.is_beta_history():
            item = self.content_item_by_attributes(hid=2, state="ok")
            item.wait_for_present()
            item.title.wait_for_and_click()
        else:
            self.history_panel_wait_for_hid_ok(2, allowed_force_refreshes=1)
            self.history_panel_click_item_title(hid=2, wait=True)
        self.assert_item_summary_includes(2, "2 sequences")
        self.screenshot("workflow_run_simple_complete")

    @selenium_test
    @managed_history
    def test_runtime_parameters_simple(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_RUNTIME_PARAMETER_SIMPLE)
        self.tool_parameter_div("num_lines")
        self.screenshot("workflow_run_runtime_parameters_initial")
        self._set_num_lines_to_3("num_lines")
        self.screenshot("workflow_run_runtime_parameters_modified")
        self.workflow_run_submit()

        self._assert_has_3_lines_after_run(hid=2)

    @selenium_test
    @managed_history
    def test_subworkflows_simple(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_NESTED_SIMPLE)
        self.components.workflow_run.subworkflow_step_icon.wait_for_visible()
        self.screenshot("workflow_run_nested_collapsed")
        self.components.workflow_run.subworkflow_step_icon.wait_for_and_click()
        self.screenshot("workflow_run_nested_open")

    @selenium_test
    @managed_history
    def test_subworkflow_runtime_parameters(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_NESTED_RUNTIME_PARAMETER)
        self.components.workflow_run.subworkflow_step_icon.wait_for_visible()
        self.screenshot("workflow_run_nested_parameters_collapsed")
        self.components.workflow_run.subworkflow_step_icon.wait_for_and_click()
        self.screenshot("workflow_run_nested_parameters_open")
        self._set_num_lines_to_3("1|num_lines")
        self.workflow_run_submit()

        self._assert_has_3_lines_after_run(hid=2)

    @selenium_test
    @managed_history
    def test_replacement_parameters(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_RENAME_ON_REPLACEMENT_PARAM)
        self.screenshot("workflow_run_rename_simple_empty")
        self._set_replacement_parameter("replaceme", "moocow")
        self.screenshot("workflow_run_rename_simple_input")
        self.workflow_run_submit()
        output_hid = 2
        if self.is_beta_history():
            self.content_item_by_attributes(hid=output_hid, state="ok").wait_for_present()
        else:
            self.history_panel_wait_for_hid_ok(output_hid, allowed_force_refreshes=1)
        history_id = self.current_history_id()
        details = self.dataset_populator.get_history_dataset_details(history_id, hid=output_hid)
        assert details["name"] == "moocow suffix", details

    @selenium_test
    @managed_history
    def test_step_parameter_inputs(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run("""
class: GalaxyWorkflow
inputs:
  input_int: integer
  input_data: data
steps:
  simple_constructs:
    tool_id: simple_constructs
    label: tool_exec
    in:
      inttest: input_int
      files_0|file: input_data
""")
        workflow_run = self.components.workflow_run
        input_div_element = workflow_run.input_div(label="input_int").wait_for_visible()
        input_element = input_div_element.find_element_by_css_selector("input")
        input_element.clear()
        input_element.send_keys("12345")

        self.screenshot("workflow_run_step_parameter_input")
        self.workflow_run_submit()
        output_hid = 2
        if self.is_beta_history():
            self.content_item_by_attributes(hid=output_hid, state="ok").wait_for_present()
        else:
            self.history_panel_wait_for_hid_ok(output_hid, allowed_force_refreshes=1)
        history_id = self.current_history_id()
        content = self.dataset_populator.get_history_dataset_content(history_id, hid=output_hid)
        assert "12345" in content, content
        assert "chr6_hla_hap2" in content

    @selenium_test
    @managed_history
    def test_replacement_parameters_on_subworkflows(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.open_in_workflow_run(WORKFLOW_NESTED_REPLACEMENT_PARAMETER)
        self.screenshot("workflow_run_rename_subworkflow_empty")
        self._set_replacement_parameter("replaceme", "moocow")
        self.screenshot("workflow_run_rename_subworkflow_input")
        self.workflow_run_submit()
        output_hid = 2
        if self.is_beta_history():
            self.content_item_by_attributes(hid=output_hid, state="ok").wait_for_present()
        else:
            self.history_panel_wait_for_hid_ok(output_hid, allowed_force_refreshes=1)
        history_id = self.current_history_id()
        details = self.dataset_populator.get_history_dataset_details(history_id, hid=output_hid)
        assert details["name"] == "moocow suffix", details

    @selenium_test
    def test_execution_with_tool_upgrade(self):
        name = self.workflow_upload_yaml_with_random_name(WORKFLOW_WITH_OLD_TOOL_VERSION, exact_tools=True)
        self.workflow_run_with_name(name)
        self.sleep_for(self.wait_types.UX_TRANSITION)
        # Check that this tool form contains a warning about different versions.
        self.assert_message(self.components.workflow_run.warning, contains="different versions")
        self.screenshot("workflow_run_tool_upgrade")

    @selenium_test
    @managed_history
    def test_execution_with_multiple_inputs(self):
        history_id, inputs = self.workflow_run_setup_inputs(WORKFLOW_WITH_DYNAMIC_OUTPUT_COLLECTION)
        self.open_in_workflow_run(WORKFLOW_WITH_DYNAMIC_OUTPUT_COLLECTION)
        self.workflow_run_specify_inputs(inputs)
        self.screenshot("workflow_run_two_inputs")
        self.workflow_run_submit()
        if self.is_beta_history():
            timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
            item = self.content_item_by_attributes(hid=7, state="ok")
            self.wait_for_present(item, timeout=timeout)
        else:
            self.history_panel_wait_for_hid_ok(7, allowed_force_refreshes=1)
        content = self.dataset_populator.get_history_dataset_content(history_id, hid=7)
        self.assertEqual("10.0\n30.0\n20.0\n40.0\n", content)

    @selenium_test
    @managed_history
    def test_execution_with_text_default_value_connected_to_restricted_select(self):
        self.open_in_workflow_run("""
class: GalaxyWorkflow
inputs:
  text_param:
    optional: true
    default: ex2
    restrictOnConnections: true
    type: text
steps:
  multi_select:
    tool_id: multi_select
    in:
      select_ex:
        source: text_param
""")
        element = self.components.workflow_run.input_select_field(label="text_param").wait_for_present()
        assert element.text == 'Ex2'
        self.workflow_run_submit()
        history_id = self.current_history_id()
        self.workflow_populator.wait_for_history_workflows(history_id, expected_invocation_count=1)
        content = self.dataset_populator.get_history_dataset_content(history_id, hid=1)
        assert content == 'ex2'

    @selenium_test
    @managed_history
    def test_execution_with_rules(self):
        history_id, inputs = self.workflow_run_setup_inputs(WORKFLOW_WITH_RULES_1)
        self.open_in_workflow_run(WORKFLOW_WITH_RULES_1)
        self.workflow_run_specify_inputs(inputs)
        self.screenshot("workflow_run_rules")
        self.workflow_run_submit()
        if self.is_beta_history():
            timeout = self.wait_length(self.wait_types.JOB_COMPLETION)
            item = self.content_item_by_attributes(hid=6, state="ok")
            self.wait_for_present(item, timeout=timeout)
        else:
            self.history_panel_wait_for_hid_ok(6, allowed_force_refreshes=1)
        output_content = self.dataset_populator.get_history_collection_details(history_id, hid=6)
        rules_test_data.check_example_2(output_content, self.dataset_populator)

    @selenium_test
    @managed_history
    def test_execution_with_custom_invocation_report(self):
        history_id, inputs = self.workflow_run_setup_inputs(WORKFLOW_WITH_CUSTOM_REPORT_1_TEST_DATA)
        self.open_in_workflow_run(WORKFLOW_WITH_CUSTOM_REPORT_1)
        self.workflow_run_specify_inputs(inputs)
        self.workflow_run_submit()
        self.sleep_for(self.wait_types.UX_TRANSITION)
        self.screenshot("workflow_run_invocation_report")
        self.workflow_populator.wait_for_history_workflows(history_id, expected_invocation_count=1)
        invocation_0 = self.workflow_populator.history_invocations(history_id)[0]
        self.get(f"workflows/invocations/report?id={invocation_0['id']}")
        self.wait_for_selector_visible(".embedded-dataset")
        self.screenshot("workflow_report_custom_1")

    @selenium_test
    @managed_history
    def test_execution_with_null_optional_select_from_data(self):
        self.open_in_workflow_run(WORKFLOW_SELECT_FROM_OPTIONAL_DATASET)
        self.workflow_run_submit()
        history_id = self.current_history_id()
        self.workflow_populator.wait_for_history_workflows(history_id, expected_invocation_count=1)

    def open_in_workflow_run(self, yaml_content):
        name = self.workflow_upload_yaml_with_random_name(yaml_content)
        self.workflow_run_with_name(name)

    def workflow_run_setup_inputs(self, content):
        history_id = self.current_history_id()
        yaml_content = yaml.safe_load(content)
        if "test_data" in yaml_content:
            test_data = yaml_content["test_data"]
        else:
            test_data = yaml_content
        inputs, _, _ = load_data_dict(history_id, test_data, self.dataset_populator, self.dataset_collection_populator)
        self.dataset_populator.wait_for_history(history_id)
        return history_id, inputs

    def workflow_run_specify_inputs(self, inputs):
        workflow_run = self.components.workflow_run
        for label, value in inputs.items():
            input_div_element = workflow_run.input_data_div(label=label).wait_for_visible()
            self.select2_set_value(input_div_element, "%d: " % value["hid"])

    def workflow_run_with_name(self, name):
        self.workflow_index_open()
        self.workflow_index_search_for(name)
        self.workflow_click_option(".workflow-run")

    def _assert_has_3_lines_after_run(self, hid):
        if self.is_beta_history():
            self.content_item_by_attributes(hid=hid, state="ok").wait_for_present()
        else:
            self.history_panel_wait_for_hid_ok(hid, allowed_force_refreshes=1)
        history_id = self.current_history_id()
        content = self.dataset_populator.get_history_dataset_content(history_id, hid=hid)
        assert len([x for x in content.split("\n") if x]) == 3, content

    def _set_num_lines_to_3(self, element_id):
        # for random_lines num_lines parameter as runtime parameter in workflow form.
        div = self.tool_parameter_div(element_id)
        input_element = div.find_element_by_css_selector("input")
        # runtime parameters not being set to tool default value:
        # https://github.com/galaxyproject/galaxy/pull/7157
        # initial_value = input_element.get_attribute("value")
        # assert initial_value == "1", initial_value
        input_element.clear()
        input_element.send_keys("3")

    def _set_replacement_parameter(self, element_id, value):
        # for random_lines num_lines parameter as runtime parameter in workflow form.
        div = self.tool_parameter_div(element_id)
        input_element = div.find_element_by_css_selector("input")
        initial_value = input_element.get_attribute("value")
        assert initial_value == "", initial_value
        input_element.clear()
        input_element.send_keys(value)



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class SignOutTestCase(SeleniumTestCase):
    @selenium_test
    def test_sign_out(self):
        email = self._get_random_email()
        self.register(email)
        self.click_masthead_user()
        self.components.masthead.preferences.wait_for_and_click()
        self.components.preferences.sign_out.wait_for_and_click()
        self.components.sign_out.cancel_button.wait_for_and_click()
        assert self.is_logged_in()
        new_email = self.driver.find_element_by_id("user-preferences-current-email").text
        self.assertTrue(email == new_email)
        self.components.preferences.sign_out.wait_for_and_click()
        self.components.sign_out.sign_out_button.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_TRANSITION)
        assert not self.is_logged_in()



import os

from galaxy.util import galaxy_root_path
from .framework import (
    selenium_test,
    SeleniumTestCase,
    TIMEOUT_MULTIPLIER,
)

STOCK_TOURS_DIRECTORY = os.path.join(galaxy_root_path, "config", "plugins", "tours")


class TestStockToursTestCase(SeleniumTestCase):

    @selenium_test
    def test_core_galaxy_ui(self):
        sleep_on_steps = {
            "Tools": 20 * TIMEOUT_MULTIPLIER,
            "Select a tool": 2 * TIMEOUT_MULTIPLIER,
            "History": 20 * TIMEOUT_MULTIPLIER,
        }
        self.run_tour(
            os.path.join(STOCK_TOURS_DIRECTORY, "core.galaxy_ui.yaml"),
            sleep_on_steps=sleep_on_steps
        )

    @selenium_test
    def test_core_scratchbook(self):
        self.run_tour(
            os.path.join(STOCK_TOURS_DIRECTORY, "core.scratchbook.yaml"),
        )

    @selenium_test
    def test_core_history(self):
        self.run_tour(
            os.path.join(STOCK_TOURS_DIRECTORY, "core.history.yaml"),
        )



from .framework import (
    selenium_test,
    SeleniumTestCase,
    UsesLibraryAssertions
)


class LibraryToCollectionsTestCase(SeleniumTestCase, UsesLibraryAssertions):

    requires_admin = True

    @selenium_test
    def test_library_collection_export_new_history(self):
        self.collection_export(is_new_history=True)

    @selenium_test
    def test_library_collection_export(self):
        self.collection_export()

    @selenium_test
    def test_library_pair_export_new_history(self):
        self.collection_export(is_new_history=True, collection_option="paired")

    @selenium_test
    def test_library_pair_export(self):
        self.collection_export(collection_option="paired")

    @selenium_test
    def test_export_pairs_list_new_history(self):
        self.list_of_pairs_export(is_new_history=True)

    @selenium_test
    def test_export_pairs_list(self):
        self.list_of_pairs_export()

    def prepare_library_for_data_export(self, files_to_import, history_name=None):
        self.create_new_library()
        self.home()
        self.history_panel_create_new()
        self.libraries_open_with_name(self.name)
        self.assert_num_displayed_items_is(0)
        self.populate_library_folder_from_import_dir(self.name, files_to_import)
        for i in range(len(files_to_import)):
            self.components.libraries.folder.select_dataset(rowindex=i + 1).wait_for_and_click()
        self.components.libraries.folder.add_to_history.wait_for_and_click()
        self.components.libraries.folder.add_to_history_collection.wait_for_and_click()
        if history_name is not None:
            self.components.libraries.folder.export_to_history_new_history.wait_for_and_send_keys(history_name)

    def build_collection_and_assert(self):
        self.collection_builder_set_name(self._get_random_name())
        self.collection_builder_create()
        self.home()
        self.history_panel_wait_for_hid_ok(3)

    def collection_export(self, is_new_history=False, collection_option=None):
        random_name = self._get_random_name()

        self.prepare_library_for_data_export(["1.bam", "1.bed"], random_name if is_new_history else None)
        self.components.libraries.folder.export_to_history_options.wait_for_and_click()

        if collection_option is not None:
            self.components.libraries.folder.export_to_history_paired_option(
                collection_option=collection_option).wait_for_and_click()
        self.screenshot(f'libraries_to_collection_landing_is_new_history={is_new_history}')
        self.components.libraries.folder.import_datasets_ok_button.wait_for_and_click()
        self.build_collection_and_assert()
        if is_new_history:
            assert self.history_panel_name_element().text == random_name

    def list_of_pairs_export(self, is_new_history=False):
        history_name = self._get_random_name()
        self.prepare_library_for_data_export(["bam_from_sam.bam", "asian_chars_1.txt", "1.bam", "1.bed"],
                                             history_name if is_new_history else None)
        self.components.libraries.folder.export_to_history_options.wait_for_and_click()
        self.components.libraries.folder.export_to_history_paired_option(collection_option="list:paired").wait_for_and_click()
        self.screenshot(f'test_export_pairs_list={is_new_history}')
        self.components.libraries.folder.import_datasets_ok_button.wait_for_and_click()
        self.components.libraries.folder.clear_filters.wait_for_and_click()
        self.collection_builder_click_paired_item("forward", 0)
        self.collection_builder_click_paired_item("reverse", 1)
        self.components.libraries.folder.export_to_history_collection_name.wait_for_and_send_keys(self._get_random_name())
        self.collection_builder_create()
        self.home()
        self.history_panel_wait_for_hid_ok(3)
        self.history_panel_wait_for_hid_hidden(1)
        self.history_panel_wait_for_hid_hidden(2)
        if is_new_history:
            assert self.history_panel_name_element().text == history_name



import json

import pytest

from galaxy.selenium.navigates_galaxy import retry_call_during_transitions
from galaxy_test.base import rules_test_data
from galaxy_test.base.populators import (
    flakey,
    skip_if_github_down,
    stage_rules_example,
)
from .framework import (
    managed_history,
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase,
    UsesHistoryItemAssertions,
)


class ToolFormTestCase(SeleniumTestCase, UsesHistoryItemAssertions):

    @selenium_test
    def test_run_tool_verify_contents_by_peek(self):
        self._run_environment_test_tool()

        self.history_panel_wait_for_hid_ok(1)
        self.history_panel_click_item_title(hid=1)
        self.assert_item_peek_includes(1, "42")

    @selenium_test
    def test_run_tool_verify_dataset_details(self):
        self._run_environment_test_tool()
        self.history_panel_wait_for_hid_ok(1)
        self._check_dataset_details_for_inttest_value(1)

    @selenium_test
    def test_verify_dataset_details_tables(self):
        self._run_environment_test_tool()
        self.history_panel_wait_for_hid_ok(1)

        hda = self.latest_history_item()
        self._check_dataset_details_for_inttest_value(1)

        dataset_details_key_value_pairs = self._table_to_key_value_elements("table#dataset-details")
        number_found = name_found = format_found = False
        for key, value in dataset_details_key_value_pairs:
            if "Number" in key.text:
                assert str(hda["hid"]) in value.text
                number_found = True
            if "Name" in key.text:
                assert hda["name"] in value.text
                name_found = True
            if "Format" in key.text:
                assert hda["extension"] in value.text
                format_found = True

        assert number_found
        assert name_found
        assert format_found

        job_outputs = self._table_to_key_value_elements("table#job-outputs")
        assert job_outputs[0][0].text == 'environment_variables'
        assert "View data\nEdit attributes\nDelete" in job_outputs[0][1].text
        job_outputs[0][1].click()
        assert job_outputs[0][1].find_element_by_css_selector('pre').text == '42\nmoo\nNOTTHREE'
        dataset_operations = self.wait_for_selector_visible("table#job-outputs div.dropdown")
        dataset_operations.click()
        menu = self.wait_for_selector_visible("table#job-outputs div.dropdown ul")
        self.click_menu_item(menu, 'Run job again')
        self.components.tool_form.execute.wait_for_visible()

    @staticmethod
    def click_menu_item(menu, text):
        for element in menu.find_elements_by_css_selector('a'):
            if element.text == text:
                return element.click()

    def _table_to_key_value_elements(self, table_selector):
        tool_parameters_table = self.wait_for_selector_visible(table_selector)
        tbody_element = tool_parameters_table.find_element_by_css_selector("tbody")
        trs = tbody_element.find_elements_by_css_selector("tr")
        assert trs
        key_value_pairs = []
        for tr in trs:
            tds = tr.find_elements_by_css_selector("td")
            assert tds
            key_value_pairs.append((tds[0], tds[1]))

        return key_value_pairs

    @selenium_test
    def test_rerun(self):
        self._run_environment_test_tool()
        self.history_panel_wait_for_hid_ok(1)
        self.hda_click_primary_action_button(1, "rerun")

        def check_recorded_val():
            inttest_div_element = self.tool_parameter_div("inttest")
            inttest_input_element = inttest_div_element.find_element_by_css_selector("input")
            recorded_val = inttest_input_element.get_attribute("value")
            # Assert form re-rendered with correct value in textbox.
            assert recorded_val == "42", recorded_val

        # These form entries seem to be replaced/updated occasionally
        # causing stale elements.
        retry_call_during_transitions(check_recorded_val)
        self.tool_form_execute()

        self.history_panel_wait_for_hid_ok(2)
        self._check_dataset_details_for_inttest_value(2)

    @selenium_test
    @flakey
    def test_run_data(self):
        test_path = self.get_filename("1.fasta")
        test_path_decoy = self.get_filename("1.txt")
        # Upload form posts bad data if executed two times in a row like this, so
        # wait between uploads. xref https://github.com/galaxyproject/galaxy/issues/5169
        self.perform_upload(test_path)
        self.history_panel_wait_for_hid_ok(1)
        self.perform_upload(test_path_decoy)
        self.history_panel_wait_for_hid_ok(2)

        self.home()
        self.tool_open("head")
        self.tool_set_value("input", "1.fasta", expected_type="data")

        self.screenshot("tool_form_simple_data")
        self.tool_form_execute()

        self.history_panel_wait_for_hid_ok(3)

        latest_hda = self.latest_history_item()
        assert latest_hda["hid"] == 3
        assert latest_hda["name"] == "Select first on data 1"

    @selenium_test
    def test_bibtex_rendering(self):
        self.home()
        # prefetch citations so they will be available quickly when rendering tool form.
        citations_api = self.api_get("tools/bibtex/citations")
        citation_count = len(citations_api)
        self.tool_open("bibtex")
        self.components.tool_form.about.wait_for_and_click()

        @retry_assertion_during_transitions
        def assert_citations_visible():
            references = self.components.tool_form.reference.all()
            assert len(references) == citation_count
            return references

        references = assert_citations_visible()
        doi_resolved_citation = references[0]
        assert "platform for interactive" in doi_resolved_citation.text
        self.screenshot("tool_form_citations_formatted")

    def _check_dataset_details_for_inttest_value(self, hid, expected_value="42"):
        self.hda_click_details(hid)
        self.components.dataset_details._.wait_for_visible()
        tool_parameters_table = self.components.dataset_details.tool_parameters.wait_for_visible()
        tbody_element = tool_parameters_table.find_element_by_css_selector("tbody")
        tds = tbody_element.find_elements_by_css_selector("td")
        assert tds
        assert any(expected_value in td.text for td in tds)

    def _run_environment_test_tool(self, inttest_value="42"):
        self.home()
        self.tool_open("environment_variables")
        self.tool_set_value("inttest", inttest_value)
        self.tool_form_execute()


class LoggedInToolFormTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_run_apply_rules_1(self):
        self._apply_rules_and_check(rules_test_data.EXAMPLE_1)
        self.screenshot("tool_apply_rules_example_1_final")

    @selenium_test
    def test_run_apply_rules_2(self):
        self._apply_rules_and_check(rules_test_data.EXAMPLE_2)
        self.screenshot("tool_apply_rules_example_2_final")

    @selenium_test
    def test_run_apply_rules_3(self):
        self._apply_rules_and_check(rules_test_data.EXAMPLE_3)
        self.screenshot("tool_apply_rules_example_3_final")

    @selenium_test
    def test_run_apply_rules_4(self):
        self._apply_rules_and_check(rules_test_data.EXAMPLE_4)
        self.screenshot("tool_apply_rules_example_4_final")

    @selenium_test
    @managed_history
    @skip_if_github_down
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_run_apply_rules_tutorial(self):
        self.home()
        self.upload_rule_start()
        self.upload_rule_set_data_type("Collection")
        self.components.upload.rule_source_content.wait_for_and_send_keys("""https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/treated1fb.txt treated_single_1
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/treated2fb.txt treated_paired_2
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/treated3fb.txt treated_paired_3
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/untreated1fb.txt untreated_single_4
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/untreated2fb.txt untreated_single_5
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/untreated3fb.txt untreated_paired_6
https://raw.githubusercontent.com/jmchilton/galaxy/apply_rules_tutorials/test-data/rules/untreated4fb.txt untreated_paired_7
""")
        self.screenshot("rules_apply_rules_example_4_1_input_paste")
        self.upload_rule_build()
        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()
        self.rule_builder_set_mapping("url", "A")
        self.rule_builder_set_mapping("list-identifiers", ["B"])
        self.rule_builder_set_collection_name("flat_count_list")
        self.rule_builder_set_extension("txt")

        self.screenshot("rules_apply_rules_example_4_2_input_rules")
        rule_builder.main_button_ok.wait_for_and_click()
        self.history_panel_wait_for_hid_ok(1)
        self.screenshot("rules_apply_rules_example_4_3_input_ready")
        self.history_multi_view_display_collection_contents(1, "list")
        self.screenshot("rules_apply_rules_example_4_4_input_list")
        self.home()
        add_depth_rules = {
            "rules": [
                {
                    "type": "add_column_metadata",
                    "value": "identifier0",
                },
                {
                    "type": "add_column_regex",
                    "target_column": 0,
                    "expression": "(.*)_(.*)_.*",
                    "group_count": 2
                }
            ],
            "mapping": [
                {
                    "type": "list_identifiers",
                    "columns": [1, 2, 0],
                }
            ],
        }
        self._tool_apply_with_source(
            add_depth_rules,
            hid=1,
            landing_screenshot="rules_apply_rules_example_4_5_apply_rules_landing",
            rule_init_screenshot="rules_apply_rules_example_4_6_apply_rules_init_flat",
            rule_complete_screenshot="rules_apply_rules_example_4_7_apply_rules_add_depth"
        )
        self.history_panel_wait_for_hid_ok(16)
        self.history_multi_view_display_collection_contents(16, "list:list:list")
        self.screenshot("rules_apply_rules_example_4_8_nested")
        self.home()
        invert_rules = {
            "rules": [
                {
                    "type": "add_column_metadata",
                    "value": "identifier0",
                },
                {
                    "type": "add_column_metadata",
                    "value": "identifier1",
                },
                {
                    "type": "add_column_metadata",
                    "value": "identifier2",
                },
            ],
            "mapping": [
                {
                    "type": "list_identifiers",
                    "columns": [1, 0, 2],
                }
            ],
        }
        self._tool_apply_with_source(
            invert_rules,
            rule_init_screenshot="rules_apply_rules_example_4_9_apply_rules_init_nested",
            rule_complete_screenshot="rules_apply_rules_example_4_10_apply_rules_inverted"
        )
        self.history_panel_wait_for_hid_ok(24)
        self.history_multi_view_display_collection_contents(24, "list:list:list")
        self.screenshot("rules_apply_rules_example_4_11_inverted")
        self.home()
        filter_rules = {
            "rules": [
                {
                    "type": "add_column_metadata",
                    "value": "identifier0",
                },
                {
                    "type": "add_filter_regex",
                    "target_column": 0,
                    "expression": ".*_single_.*",
                    "invert": False,
                }
            ],
            "mapping": [
                {
                    "type": "list_identifiers",
                    "columns": [0],
                }
            ],
        }
        self._tool_apply_with_source(
            filter_rules,
            hid=1,
            rule_complete_screenshot="rules_apply_rules_example_4_12_apply_rules_filter"
        )
        self.history_panel_wait_for_hid_ok(28)
        self.history_multi_view_display_collection_contents(28, "list")
        self.screenshot("rules_apply_rules_example_4_13_filtered")
        self.home()
        filter_and_nest_rules = {
            "rules": [
                {
                    "type": "add_column_metadata",
                    "value": "identifier0",
                },
                {
                    "type": "add_filter_regex",
                    "target_column": 0,
                    "expression": ".*_single_.*",
                    "invert": False,
                },
                {
                    "type": "add_column_regex",
                    "target_column": 0,
                    "expression": "(.*)_single_.*",
                    "group_count": 1
                }
            ],
            "mapping": [
                {
                    "type": "list_identifiers",
                    "columns": [1, 0],
                }
            ],
        }
        self._tool_apply_with_source(
            filter_and_nest_rules,
            hid=1,
            rule_complete_screenshot="rules_apply_rules_example_4_14_apply_rules_filtered_and_nested"
        )
        self.history_panel_wait_for_hid_ok(32)
        self.history_multi_view_display_collection_contents(32, "list:list")
        self.screenshot("rules_apply_rules_example_4_15_filtered_and_nested")

    def _apply_rules_and_check(self, example):
        rule_builder = self.components.rule_builder

        self.home()
        history_id = self.current_history_id()
        stage_rules_example(self.api_interactor_for_logged_in_user(), history_id, example)
        self.dataset_populator.wait_for_history(history_id)
        self.home()
        self._tool_open_apply_rules()
        self.screenshot("tool_apply_rules_landing")
        self.tool_parameter_edit_rules()
        rule_builder._.wait_for_visible()
        self.screenshot("tool_apply_rules_builder_landing")
        self.rule_builder_set_source(json.dumps(example["rules"]))
        self.screenshot("tool_apply_rules_after")
        rule_builder.main_button_ok.wait_for_and_click()
        self.tool_form_execute()
        output_hid = example["output_hid"]
        self.home()
        self.history_panel_wait_for_hid_ok(output_hid)
        output_hdca = self.dataset_populator.get_history_collection_details(history_id, hid=output_hid, wait=False)
        example["check"](output_hdca, self.dataset_populator)

    def _tool_apply_with_source(self, rules_json, hid=None, landing_screenshot=None, rule_init_screenshot=None, rule_complete_screenshot=None):
        self._tool_open_apply_rules()
        if hid:
            self.tool_set_value("input", f"{hid}:", expected_type="data_collection")
        if landing_screenshot:
            self.screenshot(landing_screenshot)
        rule_builder = self.components.rule_builder
        self.tool_parameter_edit_rules()
        rule_builder._.wait_for_visible()
        if rule_init_screenshot:
            self.screenshot(rule_init_screenshot)
        self.rule_builder_set_source(json.dumps(rules_json))
        if rule_complete_screenshot:
            self.screenshot(rule_complete_screenshot)
        rule_builder.main_button_ok.wait_for_and_click()
        self.tool_form_execute()

    def _tool_open_apply_rules(self):
        self.tool_open("__APPLY_RULES__", outer=True)  # may appear twice in panel, grab top-level link



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class AnonymousHistoriesTestCase(SeleniumTestCase):

    @selenium_test
    def test_anon_history_landing(self):
        self.home()
        self.assert_initial_history_panel_state_correct()

        # Anonymous users can annotate or tag, these components should be absent.
        self.components.history_panel.tag_icon.assert_absent_or_hidden()
        self.components.history_panel.annotation_icon.assert_absent_or_hidden()

        # History has a name but...
        name_element = self.components.history_panel.name.wait_for_and_click()

        # ... name should NOT be editable when clicked by anon-user
        editable_text_class = self.navigation._.selectors.editable_text
        assert editable_text_class.as_css_class not in name_element.get_attribute("class")

    @selenium_test
    def test_anon_history_upload(self):
        self.home()
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        # Reload the history and make sure the state is preserved.
        self.home()
        self.history_panel_wait_for_hid_state(1, "ok")

        # empty should be NO LONGER be displayed
        self.components.history_panel.empty_message.assert_absent_or_hidden()

    @selenium_test
    def test_anon_history_after_registration(self):
        self._upload_file_anonymous_then_register_user()
        self.home()
        self.history_panel_wait_for_hid_state(1, "ok")

    @selenium_test
    def test_clean_anon_history_after_logout(self):
        self._upload_file_anonymous_then_register_user()
        self.logout_if_needed()
        # Give Galaxy the chance to load a new empty history for that now
        # anonymous user. Make sure this new history is empty.
        self.home()
        self.history_panel_wait_for_history_loaded()
        history_contents = self.history_contents()
        assert len(history_contents) == 0

    def _upload_file_anonymous_then_register_user(self):
        self.home()
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.register()



import pytest

from galaxy_test.base.populators import flakey
from .framework import (
    selenium_test,
    SeleniumTestCase
)


class HistoryPanelPaginationTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    @flakey  # The next button doesn't always work - maybe a delay in JS callback registering for that.
    def test_pagination(self):
        if self.is_beta_history():
            raise pytest.skip("There is no pagination on the beta history panel")

        history_id = self.current_history_id()

        self.dataset_populator.new_dataset(history_id, content='1\t2\t3', name="data1")
        self.dataset_populator.new_dataset(history_id, content='2\t3\t4', name="data2")
        self.dataset_populator.new_dataset(history_id, content='3\t4\t5', name="data3")
        self.dataset_populator.new_dataset(history_id, content='4\t5\t6', name="data4")
        self.dataset_populator.new_dataset(history_id, content='5\t6\t7', name="data5")

        self.home()
        for hid in [1, 2, 3, 4, 5]:
            self.history_panel_wait_for_hid_state(hid, "ok")

        with self.local_storage("historyContentsLimitPerPageDefault", 3):
            self.home()
            self.history_panel_wait_for_hid_state(5, "ok")
            self.screenshot("history_panel_pagination_initial")
            pagination_option_text = self.components.history_panel.pagination_pages_selected_option.wait_for_text()
            assert "1st of 2 pages" in pagination_option_text
            self.components.history_panel.pagination_pages.wait_for_and_click()
            self.screenshot("history_panel_pagination_pages_drop_down")
            self.components.history_panel.pagination_next.wait_for_and_click()
            self.sleep_for(self.wait_types.UX_TRANSITION)
            self.screenshot("history_panel_pagination_second")
            pagination_option_text = self.components.history_panel.pagination_pages_selected_option.wait_for_text()
            assert "2nd of 2 pages" in pagination_option_text
            self.components.history_panel.pagination_previous.wait_for_and_click()

            self.sleep_for(self.wait_types.UX_TRANSITION)
            pagination_option_text = self.components.history_panel.pagination_pages_selected_option.wait_for_text()
            assert "1st of 2 pages" in pagination_option_text



"""Examples and framework helpers for driving Galaxy with Selenium in Jupyter.

Example Notebooks
-----------------

There are a couple example notebooks in this directory that demonstrate two different
helper classes that provide a Python context for interacting with Galaxy.

``notebook_example_library.ipynb`` demonstrates using :class:`galaxy.selenium.jupyter_context.JupyterContext`,
an interface that only depends on the pip installable library ``galaxy-selenium``.
``JupyterContext`` inherits :class:`galaxy.selenium.navigates_galaxy.NavigatesGalaxy`
and :class:`galaxy.selenium.has_driver.HasDriver` and contains numerous functions to
interact with Galaxy in generic terms using Selenium.

``notebook_example_testing.ipynb`` demonstrates using :class:`galaxy_test.selenium.jupyter_context.JupyterTestContextImpl`,
an interface that extends ``JupyterContextImpl`` to additionally provide testing framework
populators (:mod:`galaxy_test.base.populators`) configured to work with the same
user as is logged in with Selenium and that provide a quick way to populate the target
Galaxy with testing fixtures.

The first example notebook is good for prototyping stand-alone applications that drive
Galaxy, the second example notebook is setup to mirror the environment available to Selenium
test cases that ship with Galaxy (e.g. :class:`galaxy_test.selenium.framework:SeleniumTestCase`)
and can be used for interactively developing Selenium tests for Galaxy. Additionally, it shows how can ActionChains
could be accessed within Jupyter notebook.

These notebooks start with a cell that simply defines a ``config`` variable in the Python
environment on a cell that is marked with metadata indicated it is "parameters". This
isn't required at all to use the notebook iteratively, but it does provide a nice hook
for executing the notebook in a parameterized way using papermill_ after its creation.

The second cell in both of these notebooks setup the Galaxy interaction context - either
a :class:`galaxy.selenium.jupyter_context.JupyterContextImpl` or
:class:`galaxy_test.selenium.jupyter_context.JupyterTestContextImpl` depending on
the notebook. These cells use a function named ``init`` to create the desired
interaction context (called ``gx_selenium_context`` in both cases).

::

    from galaxy.selenium.jupyter_context import init
    gx_selenium_context = init(config)

The optional ``config`` dictionary can be used to configure both Selenium and the Galaxy
that is targeted. If ``config`` is none and the working directory Jupyter is executing
inside of has a file named ``galaxy_selenium_context.yml`` - ``init`` will read that file
and use it as the target configuration.

Both contexts can be configured with the following dictionary keys (either from
``galaxy_selenium_context.yml`` or using papermill_).

``local_galaxy_url``
    Galaxy URL to target during execution.

``selenium_galaxy_url``
    Galaxy URL relative to the Selenium being used. This doesn't need to be set unless
    you're using a remote Selenium server (i.e one not running directly on ``localhost``).

``driver``
    If set, this should be a dictionary describing variables to send to
    :class:`galaxy.selenium.driver_factory.ConfiguredDriver` and define how to
    setup the Selenium client used. Useful keys and their defaults include -
    ``browser`` (``"auto"``), ``remote`` (``false``), ``remote_host`` (``"127.0.0.1"``),
    ``remote_port`` (``4444``), ``headless`` (``false``).

Additionally, the testing context also allows additional configuration variables used
for populating test fixtures (with populators) or by extra testing helpers (e.g.
``gx_selenium_context.test_login()``).

``admin_api_key``
    An API key to be used for API interactions that require an admin account.

``login_email``
    A test account login or email address to be used for tests that require a logged
    in user.

``login_password``
    The password to be used for the account referenced by ``login_email``.

.. note:: Using Papermill

    ::

        $ . venv/bin/activate  # first two commands only needed first time
        $ pip install papermill
        $ papermill galaxy_test/selenium/jupyter/notebook_example_library.ipynb  galaxy_test/selenium/jupyter/notebook_example_library_output.ipynb -y "
        config: {local_galaxy_url: 'http://localhost:8081/'}
        "

Developing Galaxy Test Cases Interactively with Jupyter
-------------------------------------------------------

The Setup
~~~~~~~~~

Writing Selenium tests for Galaxy often involves annotating the Galaxy DOM with
appropriate ``class`` and ``data-*`` attributes, so it is best to have webpack monitor
``client/src`` and rebuild as things changes and then disable building the client
as Galaxy starts. This can be done by executing the follow two commands in separate
terminal sessions.

::

    $ GALAXY_SKIP_CLIENT_BUILD=1 GALAXY_RUN_WITH_TEST_TOOLS=1 sh run.sh
    $ make client-watch

Copy ``lib/galaxy_test/selenium/jupyter/galaxy_selenium.yml.sample`` to
``lib/galaxy_test/selenium/jupyter/galaxy_selenium.yml`` and specify a
``login_email``, ``login_password``, ``admin_api_key`` to point at values
suitable for your local test environment.

::

    $ cp lib/galaxy_test/selenium/jupyter/galaxy_selenium.yml.sample lib/galaxy_test/selenium/jupyter/galaxy_selenium.yml
    $ vi lib/galaxy_test/selenium/jupyter/galaxy_selenium.yml

Start Jupyter for interactive development. This requires Jupyter to be
available in Galaxy's virtual environment.

::

    $ . venv/bin/activate  # first two commands only needed first time
    $ pip install jupyter
    $ make serve-selenium-notebooks

Filling out Tests
~~~~~~~~~~~~~~~~~~~~

Open (or copy and open) the notebook ``notebook_example_testing.ipynb`` and
start developing the test in the third notebook cell. This cell will land up
being the body a test case.

Use ``gx_selenium_context`` to navigate Galaxy, setup test conditions, and
verify expected behavior. Change ``navigation.yml`` to implement components and
``client/src`` to annotate the source - these changes will be picked up automatically
and you can simply re-run the notebook to redo the modified or extended test.
If Python test code needs to be changed (e.g. helper methods are added to
:class:`galaxy.selenium.navigates_galaxy.NavigatesGalaxy`), you'll need to
select "Restart & Run All" from the "Kernel" menu of Jupyter to pickup the new
changes.


Once the test is complete, copy all the cells after the initialization into a
new test case file (either in ``lib/galaxy_test/selenium`` or
``test/integration_selenium``). Some examples of pull request commits developed this
way include Galaxy's initial `workflow invocation detail tests
<https://github.com/galaxyproject/galaxy/pull/11160/commits/b6d99320d99e93f812adc0df494447525978ebfa>`__
and initial `history import and export tests <https://github.com/galaxyproject/galaxy/pull/11152/files>`__.
These commits demonstrate the basics of how to turn operations on a ``gx_selenium_context``
into a Galaxy test case, how to add UI selectors to ``navigation.yml``, how to add
functionality to ``navigates_galaxy.py`` to be shared across test cases.
The `history import and export tests <https://github.com/galaxyproject/galaxy/pull/11152/files>`__
additionally are examples of both regular Selenium test cases as well as their
integration-style variant that configures Galaxy before the test runs.

Much more information on structuring Selenium test cases generally, the difference
between tests that should be place in ``lib/galaxy_test/selenium`` versus ones
that should be placed in ``test/integration_selenium``, and information on running
Selenium test cases can all be found in :doc:`../dev/writing_tests`.

.. note::

    Test Case Attributes

    If you'd like to simplify your test by logging into Galaxy as a pre-condition,
    add ``ensure_registered = True`` on the test case class. Your test case can
    use an admin key to populate fixtures without any extra annotation but to ensure
    a user logged in with ``ensure_registered`` is an admin user simply annotate
    your test case with ``requires_admin = True``.

.. _papermill: https://papermill.readthedocs.io/en/latest/

"""



from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SharedStateSeleniumTestCase,
)


class SavedHistoriesTestCase(SharedStateSeleniumTestCase):

    @selenium_test
    def test_histories_list(self):
        self._login()
        self.navigate_to_histories_page()
        self.assert_histories_in_grid([self.history2_name, self.history3_name])

    @selenium_test
    def test_history_switch(self):
        self._login()
        self.navigate_to_histories_page()
        self.screenshot("histories_saved_grid")
        self.click_grid_popup_option(self.history2_name, 'Switch')
        self.sleep_for(self.wait_types.UX_RENDER)

        @retry_assertion_during_transitions
        def assert_history_name_switched():
            self.assertEqual(self.history_panel_name(), self.history2_name)

        assert_history_name_switched()

    @selenium_test
    def test_history_view(self):
        self._login()
        self.navigate_to_histories_page()
        self.click_grid_popup_option(self.history2_name, 'View')
        history_name = self.wait_for_selector('.name.editable-text')
        self.assertEqual(history_name.text, self.history2_name)

    @selenium_test
    def test_history_publish(self):
        self._login()
        self.navigate_to_histories_page()

        # Publish the history
        self.click_grid_popup_option(self.history2_name, 'Share or Publish')
        self.make_accessible_and_publishable()

        self.navigate_to_histories_page()

        self.histories_click_advanced_search()
        self.select_filter('sharing', 'published')
        self.sleep_for(self.wait_types.UX_RENDER)

        self.assert_histories_in_grid([self.history2_name])

    @selenium_test
    def test_rename_history(self):
        self._login()
        self.navigate_to_histories_page()

        self.click_grid_popup_option('Unnamed history', 'Rename')

        # Rename the history
        history_name_input = self.wait_for_selector('.ui-form-element input.ui-input')
        history_name_input.clear()
        history_name_input.send_keys(self.history1_name)

        self.wait_for_and_click_selector("button#submit")

        self.navigate_to_histories_page()

        self.assert_histories_in_grid([self.history1_name, self.history2_name, self.history3_name])

    @selenium_test
    def test_delete_and_undelete_history(self):
        self._login()
        self.navigate_to_histories_page()

        # Delete the history
        self.click_grid_popup_option(self.history2_name, 'Delete')

        self.assert_histories_in_grid([self.history2_name], False)

        self.histories_click_advanced_search()
        self.select_filter('deleted', 'True')
        self.sleep_for(self.wait_types.UX_RENDER)

        # Restore the history
        self.click_grid_popup_option(self.history2_name, 'Undelete')

        self.assert_grid_histories_are([])
        self.select_filter('deleted', 'False')

        self.assert_histories_in_grid([self.history2_name])

    @selenium_test
    def test_permanently_delete_history(self):
        self._login()
        self.create_history(self.history4_name)

        self.navigate_to_histories_page()
        self.assert_histories_in_grid([self.history4_name])

        self.click_grid_popup_option(self.history4_name, 'Delete Permanently')
        alert = self.driver.switch_to.alert
        alert.accept()

        self.assert_histories_in_grid([self.history4_name], False)

        self.histories_click_advanced_search()
        self.select_filter('deleted', 'True')

        self.assert_histories_in_grid([self.history4_name])

    @selenium_test
    def test_delete_and_undelete_multiple_histories(self):
        self._login()
        self.navigate_to_histories_page()

        delete_button_selector = 'input[type="button"][value="Delete"]'
        undelete_button_selector = 'input[type="button"][value="Undelete"]'

        # Delete multiple histories
        self.check_histories([self.history2_name, self.history3_name])
        self.wait_for_and_click_selector(delete_button_selector)

        self.assert_histories_in_grid([self.history2_name, self.history3_name], False)

        self.histories_click_advanced_search()
        self.select_filter('deleted', 'True')
        self.sleep_for(self.wait_types.UX_RENDER)

        # Restore multiple histories
        self.check_histories([self.history2_name, self.history3_name])
        self.wait_for_and_click_selector(undelete_button_selector)

        self.assert_grid_histories_are([])
        # Following msg popups but goes away and so can cause transient errors.
        # self.wait_for_selector_visible('.donemessage')
        self.select_filter('deleted', 'False')

        self.assert_histories_in_grid([self.history2_name, self.history3_name])

    @selenium_test
    def test_sort_by_name(self):
        self._login()
        self.navigate_to_histories_page()

        self.wait_for_and_click_selector('.sort-link[sort_key="name"]')
        actual_histories = self.get_histories()

        expected_histories = [self.history2_name, self.history3_name]
        if self.history1_name in actual_histories:
            expected_histories.append(self.history1_name)
        expected_histories = sorted(expected_histories)

        # Filter out histories created by other tests
        actual_histories = [x for x in actual_histories if x in self.all_histories]

        self.assertEqual(actual_histories, expected_histories)

    @selenium_test
    def test_standard_search(self):
        self._login()
        self.navigate_to_histories_page()

        search_input = self.components.grids.free_text_search.wait_for_visible()
        search_input.send_keys(self.history2_name)
        self.send_enter(search_input)

        self.sleep_for(self.wait_types.UX_RENDER)

        self.assert_grid_histories_are([self.history2_name])

        self.unset_filter('free-text-search', self.history2_name)
        search_input = self.components.grids.free_text_search.wait_for_visible()
        search_input.send_keys(self.history4_name)
        self.send_enter(search_input)

        self.sleep_for(self.wait_types.UX_RENDER)

        self.assert_grid_histories_are([])

    @selenium_test
    def test_advanced_search(self):
        self._login()
        self.navigate_to_histories_page()

        self.histories_click_advanced_search()

        name_filter_selector = '#input-name-filter'
        tags_filter_selector = '#input-tags-filter'

        # Search by name
        self.set_filter(name_filter_selector, self.history2_name)
        self.assert_grid_histories_are([self.history2_name])
        self.unset_filter('name', self.history2_name)

        self.set_filter(name_filter_selector, self.history4_name)
        self.assert_grid_histories_are([])
        self.unset_filter('name', self.history4_name)

        # Search by tags
        self.set_filter(tags_filter_selector, self.history3_tags[0])
        self.assert_grid_histories_are([self.history3_name])
        self.unset_filter('tags', self.history3_tags[0])

        self.set_filter(tags_filter_selector, self.history4_tags[0])
        self.assert_grid_histories_are([])
        self.unset_filter('tags', self.history4_tags[0])

    @selenium_test
    def test_tags(self):
        self._login()
        self.navigate_to_histories_page()

        # Insert a tag
        tags_cell = self.get_history_tags_cell(self.history2_name)
        tag_area = tags_cell.find_element_by_css_selector('.ti-new-tag-input-wrapper input')
        tag_area.click()
        tag_area.send_keys(self.history2_tags[0])
        self.send_enter(tag_area)

        self.sleep_for(self.wait_types.UX_RENDER)

        # Search by tag
        tags_cell = self.get_history_tags_cell(self.history2_name)
        tag = tags_cell.find_element_by_css_selector('.ti-tag-center')
        tag.click()

        self.assert_grid_histories_are([self.history2_name], False)

    def _login(self):
        self.home()
        self.submit_login(self.user_email, retries=3)

    @retry_assertion_during_transitions
    def assert_grid_histories_are(self, expected_histories, sort_matters=True):
        actual_histories = self.get_histories()
        if not sort_matters:
            actual_histories = set(actual_histories)
            expected_histories = set(expected_histories)
        self.assertEqual(actual_histories, expected_histories)

    @retry_assertion_during_transitions
    def assert_histories_in_grid(self, expected_histories, present=True):
        actual_histories = self.get_histories()
        intersection = set(actual_histories).intersection(expected_histories)
        if present:
            self.assertEqual(intersection, set(expected_histories))
        else:
            self.assertEqual(intersection, set())

    def get_histories(self):
        return self.histories_get_history_names()

    def set_filter(self, selector, value):
        filter_input = self.wait_for_selector_clickable(selector)
        filter_input.send_keys(value)
        self.send_enter(filter_input)
        self.sleep_for(self.wait_types.UX_RENDER)

    def unset_filter(self, filter_key, filter_value):
        close_button_selector = 'a[filter_key="%s"][filter_val="%s"]' % \
            (filter_key, filter_value)
        self.wait_for_and_click_selector(close_button_selector)
        self.sleep_for(self.wait_types.UX_RENDER)

    def setup_shared_state(self):
        SavedHistoriesTestCase.user_email = self._get_random_email()
        SavedHistoriesTestCase.history1_name = self._get_random_name()
        SavedHistoriesTestCase.history2_name = self._get_random_name()
        SavedHistoriesTestCase.history3_name = self._get_random_name()
        SavedHistoriesTestCase.history4_name = self._get_random_name()
        SavedHistoriesTestCase.history2_tags = [self._get_random_name(len=5)]
        SavedHistoriesTestCase.history3_tags = [self._get_random_name(len=5)]
        SavedHistoriesTestCase.history4_tags = [self._get_random_name(len=5)]
        SavedHistoriesTestCase.all_histories = [self.history1_name, self.history2_name, self.history3_name]

        self.register(self.user_email)
        self.create_history(self.history2_name)
        self.create_history(self.history3_name)
        self.history_panel_add_tags(self.history3_tags)

    def create_history(self, name):
        self.home()
        self.history_panel_create_new_with_name(name)

    def select_filter(self, filter_key, filter_value):
        filter_selector = f'a[filter_key="{filter_key}"][filter_val="{filter_value}"]'
        self.wait_for_and_click_selector(filter_selector)

    def get_history_tags_cell(self, history_name):
        tags_cell = None
        grid = self.wait_for_selector('#grid-table-body')
        for row in grid.find_elements_by_tag_name('tr'):
            td = row.find_elements_by_tag_name('td')
            if td[1].text == history_name:
                tags_cell = td[4]
                break

        if tags_cell is None:
            raise AssertionError(f'Failed to find history with name [{history_name}]')

        return tags_cell

    def check_histories(self, histories):
        grid = self.wait_for_selector('#grid-table-body')
        for row in grid.find_elements_by_tag_name('tr'):
            td = row.find_elements_by_tag_name('td')
            history_name = td[1].text
            if history_name in histories:
                checkbox = td[0].find_element_by_tag_name('input')
                checkbox.click()



from galaxy_test.base.populators import skip_if_github_down
from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase,
)


class WorkflowManagementTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_import_from_url(self):
        self.workflow_index_open()
        self._workflow_import_from_url()

        table_elements = self.workflow_index_table_elements()
        assert len(table_elements) == 1

        new_workflow = table_elements[0].find_element_by_css_selector(".workflow-dropdown")
        assert 'TestWorkflow1 (imported from uploaded file)' in new_workflow.text, new_workflow.text

    @selenium_test
    def test_view(self):
        self.workflow_index_open()
        self._workflow_import_from_url()
        self.workflow_index_click_option("View")
        title = self.wait_for_selector(".page-body h3")
        assert "TestWorkflow1" in title.text
        self.screenshot("workflow_manage_view")
        # TODO: Test display of steps...

    @selenium_test
    def test_rename(self):
        self.workflow_index_open()
        self._workflow_import_from_url()
        self.workflow_index_rename("CoolNewName")

        @retry_assertion_during_transitions
        def check_name():
            name = self.workflow_index_name()
            assert 'CoolNewName' == name, name

        check_name()

    @selenium_test
    def test_download(self):
        self.workflow_index_open()
        self._workflow_import_from_url()
        # TODO: fill this test out - getting downloaded files in general through Selenium is a bit tough,
        # going through the motions though should catch a couple potential problems.
        self.workflow_index_click_option("Download")

    @selenium_test
    def test_tagging(self):
        self.workflow_index_open()
        self._workflow_import_from_url()

        self.workflow_index_click_tag_display()
        self.tagging_add(["cooltag"])

        @retry_assertion_during_transitions
        def check_tags():
            self.assertEqual(self.workflow_index_tags(), ["cooltag"])

        check_tags()
        self.screenshot("workflow_manage_tags")

    @selenium_test
    def test_index_search(self):
        self.workflow_index_open()
        self._workflow_import_from_url()
        self.workflow_index_rename("searchforthis")
        self._assert_showing_n_workflows(1)
        self.screenshot("workflow_manage_search")

        self.workflow_index_search_for("doesnotmatch")
        self._assert_showing_n_workflows(0)

        self.workflow_index_search_for()
        self._assert_showing_n_workflows(1)

        self.workflow_index_search_for("searchforthis")
        self._assert_showing_n_workflows(1)

    @retry_assertion_during_transitions
    def _assert_showing_n_workflows(self, n):
        self.assertEqual(len(self.workflow_index_table_elements()), n)

    @skip_if_github_down
    def _workflow_import_from_url(self):
        self.workflow_index_click_import()
        url = "https://raw.githubusercontent.com/galaxyproject/galaxy/release_19.09/test/base/data/test_workflow_1.ga"
        self.workflow_import_submit_url(url)



import unittest

from .framework import selenium_test, SeleniumTestCase


class ToolDescribingToursTestCase(SeleniumTestCase):

    def setUp(self):
        super().setUp()
        self.home()

    @selenium_test
    def test_generate_tour_no_data(self):
        """Ensure a tour without data is generated and pops up."""
        self._ensure_tdt_available()

        self.tool_open('environment_variables')

        self.tool_form_generate_tour()

        popover_component = self.components.tour.popover._
        popover_component.wait_for_visible()

        title = popover_component.title.wait_for_visible().text
        assert title == "environment_variables Tour", title

        # Run tool
        self.tool_form_execute()
        self.history_panel_wait_for_hid_ok(1)

    @selenium_test
    def test_generate_tour_with_data(self):
        """Ensure a tour with data populates history."""
        self._ensure_tdt_available()

        self.tool_open('md5sum')

        self.tool_form_generate_tour()

        self.history_panel_wait_for_hid_ok(1)

        popover_component = self.components.tour.popover._
        popover_component.wait_for_visible()

        title = popover_component.title.wait_for_visible().text
        assert title == "md5sum Tour", title
        self.screenshot("tool_describing_tour_0_start")

        popover_component.next.wait_for_and_click()

        self.sleep_for(self.wait_types.UX_RENDER)

        text = popover_component.content.wait_for_visible().text
        assert "Select dataset" in text, text
        self.screenshot("tool_describing_tour_1_select")

        popover_component.next.wait_for_and_click()

        self.sleep_for(self.wait_types.UX_RENDER)

        title = popover_component.title.wait_for_visible().text
        assert title == "Execute tool"
        self.screenshot("tool_describing_tour_2_execute")

        popover_component.end.wait_for_and_click()
        popover_component.wait_for_absent_or_hidden()

        # Run tool
        self.tool_form_execute()
        self.history_panel_wait_for_hid_ok(2)
        self.screenshot("tool_describing_tour_3_after_execute")

    def _ensure_tdt_available(self):
        """ Skip a test if the webhook TDT doesn't appear. """
        response = self.api_get('webhooks', raw=True)
        self.assertEqual(response.status_code, 200)
        data = response.json()
        webhooks = [x['id'] for x in data]
        if 'tour_generator' not in webhooks:
            raise unittest.SkipTest('Skipping test, webhook "Tool-Describing-Tours" doesn\'t appear to be configured.')



import os

from selenium.webdriver.support.ui import Select

from .framework import (
    retry_during_transitions,
    selenium_test,
    SeleniumTestCase,
    UsesLibraryAssertions
)


class LibraryContentsTestCase(SeleniumTestCase, UsesLibraryAssertions):

    requires_admin = True

    @selenium_test
    def test_sub_folder(self):
        def change_description(description):
            self.components.libraries.folder.edit_folder_btn.wait_for_and_click()
            self.components.libraries.folder.input_folder_description.wait_for_visible().clear()
            self.components.libraries.folder.input_folder_description.wait_for_and_send_keys(description)
            self.components.libraries.folder.save_folder_btn.wait_for_and_click()

        sub_folder_name = self._get_random_name(prefix="new_sub_folder")
        description = self._get_random_name(prefix="new_sub_folder_description")
        long_description = self._get_random_name(prefix="new_sub_folder_description", len=45)

        # create mew folder
        self.navigate_to_new_library()
        self.assert_num_displayed_items_is(0)
        self.libraries_folder_create(sub_folder_name)
        self.assert_num_displayed_items_is(1)

        # check empty folder
        new_folder_link = self.wait_for_xpath_visible(f'//a[contains(text(), "{sub_folder_name}")]')
        new_folder_link.click()

        # assert that 'empty folder message' is present
        self.components.libraries.folder.empty_folder_message.wait_for_present()

        # go one folder up
        self.components.libraries.folder.btn_open_parent_folder(folder_name=self.name).wait_for_and_click()
        # assert empty description
        self.components.libraries.folder.description_field.assert_absent_or_hidden()
        # change description
        change_description(description)
        assert description == self.components.libraries.folder.description_field.wait_for_text()
        change_description(long_description)

        # assert shrinked description

        shrinked_description = long_description[0:40]
        assert shrinked_description == self.components.libraries.folder.description_field_shrinked.wait_for_text()

    @selenium_test
    def test_import_dataset_from_history(self):
        self.admin_login()
        self.perform_upload(self.get_filename("1.txt"))
        self.wait_for_history()
        self.navigate_to_new_library(login=False)
        self.assert_num_displayed_items_is(0)
        self.sleep_for(self.wait_types.UX_RENDER)
        self.libraries_dataset_import(self.navigation.libraries.folder.labels.from_history)
        # Click the cancel button, make sure modal is hidden.
        self.wait_for_visible(self.navigation.libraries.folder.selectors.import_modal)
        self.wait_for_and_click(self.navigation.libraries.folder.selectors.import_datasets_cancel_button)
        self.wait_for_absent_or_hidden(self.navigation.libraries.folder.selectors.import_modal)

        self.libraries_dataset_import(self.navigation.libraries.folder.labels.from_history)
        # Need to select the right item on the dropdown
        self.sleep_for(self.wait_types.UX_RENDER)
        self._select_history_option("dataset_add_bulk", "Unnamed history")
        self.sleep_for(self.wait_types.UX_RENDER)
        self.libraries_dataset_import_from_history_select(["1.txt"])
        # Add
        self.sleep_for(self.wait_types.UX_RENDER)
        self.screenshot("libraries_dataset_import")
        self.libraries_dataset_import_from_history_click_ok()
        self.assert_num_displayed_items_is(1)

    @selenium_test
    def download_dataset_from_library(self):
        self.test_import_dataset_from_history()

        self.components.libraries.folder.select_one.wait_for_and_click()
        self.components.libraries.folder.download_button.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)
        folder_files = os.listdir(self.get_download_path())

        expected_filename = "selected_dataset_files.zip"
        assert expected_filename in folder_files

    @selenium_test
    def test_delete_dataset(self):
        self.test_import_dataset_from_history()

        self.sleep_for(self.wait_types.UX_RENDER)
        # assert "you must select at least one" modal
        assert self.components.libraries.folder.toast_msg.is_displayed

        self.components.libraries.folder.delete_btn.wait_for_and_click()

        self.components.libraries.folder.select_one.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)

        self.components.libraries.folder.delete_btn.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)

        self.assert_num_displayed_items_is(0)

    # Fine test locally but the upload doesn't work in Docker compose. I'd think
    # Galaxy must be running so that test-data/1.txt would work but it just doesn't
    # for some reason. https://jenkins.galaxyproject.org/job/jmchilton-selenium/79/artifact/79-test-errors/test_import_dataset_from_path2017100413221507137721/
    @selenium_test
    def test_import_dataset_from_path(self):
        self.navigate_to_new_library()
        self.assert_num_displayed_items_is(0)
        self.sleep_for(self.wait_types.UX_RENDER)

        # Click the cancel button, make sure modal is hidden.
        self.libraries_dataset_import(self.navigation.libraries.folder.labels.from_path)
        self.wait_for_visible(self.navigation.libraries.folder.selectors.import_modal)
        self.wait_for_and_click(self.navigation.libraries.folder.selectors.import_datasets_cancel_button)
        self.wait_for_absent_or_hidden(self.navigation.libraries.folder.selectors.import_modal)

        # Try again... this time actually select some paths.
        self.libraries_dataset_import(self.navigation.libraries.folder.labels.from_path)
        textarea = self.wait_for_and_click(self.navigation.libraries.folder.selectors.import_from_path_textarea)
        textarea.send_keys("test-data/1.txt")
        self.sleep_for(self.wait_types.UX_RENDER)
        self.wait_for_and_click(self.navigation.libraries.folder.selectors.import_datasets_ok_button)
        # Let the progress bar disappear...
        self.wait_for_absent_or_hidden(self.navigation.libraries.folder.selectors.import_progress_bar)
        self.assert_num_displayed_items_is(1)

        self.click_label("1.txt")
        self.wait_for_visible(self.navigation.libraries.dataset.selectors.table)
        elements = self.find_elements(self.navigation.libraries.dataset.selectors.table_rows)
        table_as_dict = {}
        for element in elements:
            row_values = element.text.split("\n")
            key = row_values[0]
            value = row_values[1]
            table_as_dict[key] = value

        assert table_as_dict["Name"] == "1.txt", table_as_dict
        assert table_as_dict["Genome build"] == "?", table_as_dict

    @selenium_test
    def test_import_dataset_from_import_dir(self):
        self.navigate_to_new_library()
        self.assert_num_displayed_items_is(0)
        filenames = ["1.axt", "1.bed", "1.bam"]
        self.populate_library_folder_from_import_dir(self.name, filenames)
        self.assert_num_displayed_items_is(len(filenames))

    @selenium_test
    def test_show_details(self):
        self.navigate_to_new_library()
        self.sleep_for(self.wait_types.UX_RENDER)
        self.components.libraries.folder.open_location_details_btn.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)
        self.components.libraries.folder.location_details_ok_btn.wait_for_and_click()
        self.screenshot("libraries_show_details")
        self.wait_for_overlays_cleared()
        self.screenshot("libraries_show_details_done")

    @retry_during_transitions
    def _select_history_option(self, select_id, label_text):
        select = Select(self.driver.find_element_by_id(select_id))
        select.select_by_visible_text(label_text)



from galaxy_test.base.workflow_fixtures import (
    WORKFLOW_WITH_OUTPUT_COLLECTION,
)
from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase
)


class WorkflowInvocationDetailsTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_job_details(self):
        gx_selenium_context = self
        history_id = gx_selenium_context.dataset_populator.new_history()
        gx_selenium_context.workflow_populator.run_workflow(WORKFLOW_WITH_OUTPUT_COLLECTION, history_id=history_id, assert_ok=True, wait=True)
        gx_selenium_context.navigate_to_invocations()
        invocations = gx_selenium_context.components.invocations
        invocations.invocations_table.wait_for_visible()

        @retry_assertion_during_transitions
        def assert_has_row():
            invocations.invocations_table_rows.wait_for_visible()
            invocation_rows = invocations.invocations_table_rows.all()
            assert len(invocation_rows) > 0
            return invocation_rows[0]

        first_row = assert_has_row()
        print(first_row)
        invocations.state_details.assert_absent()
        invocations.toggle_invocation_details.wait_for_visible()
        details = invocations.toggle_invocation_details.all()[0]
        details.click()
        invocations.state_details.wait_for_visible()

        @retry_assertion_during_transitions
        def assert_progress_steps_note_contains(text):
            assert text in invocations.progress_steps_note.wait_for_visible().text

        assert_progress_steps_note_contains("3 of 3 steps successfully scheduled.")
        assert "2 of 2 jobs complete." in invocations.progress_jobs_note.wait_for_visible().text

        invocations.input_details.wait_for_and_click()
        invocations.input_details_title(label="text_input").wait_for_visible()
        assert "Test_Dataset" in invocations.input_details_name(label="text_input").wait_for_visible().text

        invocations.steps_details.wait_for_and_click()
        assert "Step 1: text_input" in invocations.step_title(order_index="0").wait_for_visible().text
        assert "Step 2: split_up" in invocations.step_title(order_index="1").wait_for_visible().text
        assert "Step 3: paired" in invocations.step_title(order_index="2").wait_for_visible().text

        invocations.step_details(order_index="1").wait_for_and_click()
        invocations.step_job_details(order_index="1").wait_for_and_click()
        invocations.step_job_table(order_index="1").wait_for_visible()
        invocations.step_job_table_rows(order_index="1").wait_for_visible()
        job_rows = invocations.step_job_table_rows(order_index="1").all()
        assert len(job_rows) == 1

        invocations.step_job_information(order_index="1").assert_absent()
        job_rows[0].click()
        self.sleep_for(self.wait_types.UX_RENDER)
        self.screenshot("invocations_job_table")

        invocations.step_job_information(order_index="1").wait_for_visible()
        assert "collection_creates_pair" in invocations.step_job_information_tool_id(order_index="1").wait_for_visible().text

        invocations.step_output_collection(order_index="1").wait_for_and_click()
        invocations.step_output_collection_toggle(order_index="1").wait_for_and_click()
        invocations.step_output_collection_element_identifier(element_identifier="forward").wait_for_and_click()
        datatype = invocations.step_output_collection_element_datatype(order_index="1").wait_for_text()
        assert datatype == 'txt'



import os

import pytest
from selenium.webdriver.common.keys import Keys

from .framework import (
    selenium_test,
    SeleniumTestCase,
    UsesHistoryItemAssertions,
)


class UploadsTestCase(SeleniumTestCase, UsesHistoryItemAssertions):

    @selenium_test
    def test_upload_simplest(self):
        self.perform_upload(self.get_filename("1.sam"))

        self.history_panel_wait_for_hid_ok(1)
        history_contents = self.history_contents()
        history_count = len(history_contents)
        assert history_count == 1, "Incorrect number of items in history - expected 1, found %d" % history_count

        hda = history_contents[0]
        assert hda["name"] == '1.sam', hda
        assert hda["extension"] == "sam", hda

        self.history_panel_click_item_title(hid=1, wait=True)
        self.assert_item_dbkey_displayed_as(1, "?")

    @selenium_test
    def test_upload_specify_ext(self):
        self.perform_upload(self.get_filename("1.sam"), ext="txt")
        self.history_panel_wait_for_hid_ok(1)
        history_contents = self.history_contents()
        hda = history_contents[0]
        assert hda["name"] == '1.sam'
        assert hda["extension"] == "txt", hda

    @selenium_test
    def test_upload_specify_genome(self):
        self.perform_upload(self.get_filename("1.sam"), genome="hg18")
        self.history_panel_wait_for_hid_ok(1)

        self.history_panel_click_item_title(hid=1, wait=True)
        self.assert_item_dbkey_displayed_as(1, "hg18")

    @selenium_test
    def test_upload_specify_ext_all(self):
        self.perform_upload(self.get_filename("1.sam"), ext_all="txt")
        self.history_panel_wait_for_hid_ok(1)
        history_contents = self.history_contents()
        hda = history_contents[0]
        assert hda["name"] == '1.sam'
        assert hda["extension"] == "txt", hda

    @selenium_test
    def test_upload_specify_genome_all(self):
        self.perform_upload(self.get_filename("1.sam"), genome_all="hg18")
        self.history_panel_wait_for_hid_ok(1)

        self.history_panel_click_item_title(hid=1, wait=True)
        self.assert_item_dbkey_displayed_as(1, "hg18")

    @selenium_test
    def test_upload_list(self):
        self.upload_list([self.get_filename("1.tabular")], name="Test List")
        self.history_panel_wait_for_hid_ok(2)
        # Make sure modals disappeared - both List creator (TODO: upload).
        self.wait_for_selector_absent_or_hidden(".collection-creator")

        self.assert_item_name(2, "Test List")

        # Make sure source item is hidden when the collection is created.
        self.history_panel_wait_for_hid_hidden(1)

    @selenium_test
    def test_upload_pair(self):
        self.upload_list([self.get_filename("1.tabular"), self.get_filename("2.tabular")], name="Test Pair")
        self.history_panel_wait_for_hid_ok(3)
        # Make sure modals disappeared - both collection creator (TODO: upload).
        self.wait_for_selector_absent_or_hidden(".collection-creator")

        self.assert_item_name(3, "Test Pair")

        # Make sure source items are hidden when the collection is created.
        self.history_panel_wait_for_hid_hidden(1)
        self.history_panel_wait_for_hid_hidden(2)

    @selenium_test
    def test_upload_pair_specify_extension(self):
        self.upload_list([self.get_filename("1.tabular"), self.get_filename("2.tabular")], name="Test Pair", ext="txt", hide_source_items=False)
        self.history_panel_wait_for_hid_ok(5)
        self.history_panel_wait_for_hid_ok(1)

        history_contents = self.history_contents()
        hda = history_contents[0]
        assert hda["name"] == '1.tabular'
        assert hda["extension"] == "txt", hda

    @selenium_test
    def test_upload_paired_list(self):
        self.upload_paired_list([self.get_filename("1.tabular"), self.get_filename("2.tabular")], name="Test Paired List")
        self.history_panel_wait_for_hid_ok(3)
        # Make sure modals disappeared - both collection creator (TODO: upload).
        self.wait_for_selector_absent_or_hidden(".collection-creator")
        self.assert_item_name(3, "Test Paired List")

        # Make sure source items are hidden when the collection is created.
        self.history_panel_wait_for_hid_hidden(1)
        self.history_panel_wait_for_hid_hidden(2)

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_1_datasets(self):
        # Test case generated for:
        #   https://www.ebi.ac.uk/ena/data/view/PRJDA60709
        self.home()
        self.upload_rule_start()
        self.sleep_for(self.wait_types.UX_RENDER)
        self.screenshot("rules_example_1_1_rules_landing")
        self.components.upload.rule_source_content.wait_for_and_send_keys("""study_accession sample_accession    experiment_accession    fastq_ftp
PRJDA60709  SAMD00016379    DRX000475   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000770/DRR000770.fastq.gz
PRJDA60709  SAMD00016383    DRX000476   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000771/DRR000771.fastq.gz
PRJDA60709  SAMD00016380    DRX000477   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000772/DRR000772.fastq.gz
PRJDA60709  SAMD00016378    DRX000478   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000773/DRR000773.fastq.gz
PRJDA60709  SAMD00016381    DRX000479   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000774/DRR000774.fastq.gz
PRJDA60709  SAMD00016382    DRX000480   ftp.sra.ebi.ac.uk/vol1/fastq/DRR000/DRR000775/DRR000775.fastq.gz""")
        self._wait_for_upload_modal()
        self.screenshot("rules_example_1_2_paste")
        self.upload_rule_build()
        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()
        self.screenshot("rules_example_1_3_initial_rules")
        rule_builder.menu_button_filter.wait_for_and_click()
        self.screenshot("rule_builder_filters")
        rule_builder.menu_item_rule_type(rule_type="add-filter-count").wait_for_and_click()
        filter_editor = rule_builder.rule_editor(rule_type="add-filter-count")
        filter_editor_element = filter_editor.wait_for_visible()
        filter_input = filter_editor_element.find_element_by_css_selector("input[type='number']")
        filter_input.clear()
        filter_input.send_keys("1")
        self.screenshot("rules_example_1_4_filter_header")
        rule_builder.rule_editor_ok.wait_for_and_click()
        self.rule_builder_set_mapping("url", "D")
        self.rule_builder_set_mapping("name", "C", screenshot_name="rules_example_1_5_mapping_edit")
        self.screenshot("rules_example_1_6_mapping_set")
        self.rule_builder_set_extension("fastqsanger.gz")
        self.screenshot("rules_example_1_7_extension_set")
        # rule_builder.main_button_ok.wait_for_and_click()
        # self.history_panel_wait_for_hid_ok(6)
        # self.screenshot("rules_example_1_6_download_complete")

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_2_list(self):
        self.perform_upload(self.get_filename("rules/PRJDA60709.tsv"))
        self.history_panel_wait_for_hid_ok(1)
        self.upload_rule_start()
        self.upload_rule_set_data_type("Collection")
        self.upload_rule_set_input_type("History Dataset")
        self.upload_rule_set_dataset(1)
        self._wait_for_upload_modal()
        self.screenshot("rules_example_2_1_inputs")
        self.upload_rule_build()
        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()
        self.screenshot("rules_example_2_2_initial_rules")
        # Filter header.
        self.rule_builder_filter_count(1)
        self.rule_builder_set_mapping("url", "D")
        self.rule_builder_set_mapping("list-identifiers", "C")
        self.rule_builder_set_extension("fastqsanger.gz")
        self.screenshot("rules_example_2_3_rules")
        self.rule_builder_set_collection_name("PRJDA60709")
        self.screenshot("rules_example_2_4_name")
        # rule_builder.main_buton_ok.wait_for_and_click()
        # self.history_panel_wait_for_hid_ok(2)
        # self.screenshot("rules_example_2_5_download_complete")

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_3_list_pairs(self):
        self.perform_upload(self.get_filename("rules/PRJDB3920.tsv"))
        self.history_panel_wait_for_hid_ok(1)
        self.upload_rule_start()
        self.upload_rule_set_data_type("Collection")
        self.upload_rule_set_input_type("History Dataset")
        self.upload_rule_set_dataset(1)
        self._wait_for_upload_modal()
        self.screenshot("rules_example_3_1_inputs")
        self.upload_rule_build()
        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()
        self.screenshot("rules_example_3_2_initial_rules")
        # Filter header.
        self.rule_builder_filter_count(1)
        self.rule_builder_set_mapping("list-identifiers", "C")
        self.rule_builder_set_extension("fastqsanger.gz")
        self.screenshot("rules_example_3_3_old_rules")
        self.rule_builder_add_regex_groups("D", 2, r"(.*);(.*)", screenshot_name="rules_example_3_4_regex")
        self.screenshot("rules_example_3_6_with_regexes")
        # Remove A also?
        self.rule_builder_remove_columns(["D"], screenshot_name="rules_example_3_7_removed_column")
        self.rule_builder_split_columns(["D"], ["E"], screenshot_name="rules_example_3_8_split_columns")
        self.screenshot("rules_example_3_9_columns_are_split")
        self.rule_builder_add_regex_groups("D", 1, r".*_(\d).fastq.gz", screenshot_name="rules_example_3_10_regex_paired")
        self.screenshot("rules_example_3_11_has_paired_id")
        self.rule_builder_swap_columns("D", "E", screenshot_name="rules_example_3_12_swap_columns")
        self.screenshot("rules_example_3_13_swapped_columns")
        self.rule_builder_set_mapping("paired-identifier", "D")
        self.rule_builder_set_mapping("url", "E")
        self.rule_builder_set_collection_name("PRJDB3920")
        self.screenshot("rules_example_3_14_paired_identifier_set")

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_4_accessions(self):
        # http://www.uniprot.org/uniprot/?query=proteome:UP000052092+AND+proteomecomponent:%22Genome%22
        self._setup_uniprot_example()
        self._wait_for_upload_modal()
        self.screenshot("rules_example_4_1_inputs")
        self.upload_rule_build()
        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()
        self.screenshot("rules_example_4_2_initial_rules")
        self.rule_builder_filter_count(1)
        self.rule_builder_remove_columns(["B", "C", "E", "F", "G"])
        self.rule_builder_set_mapping("info", "B")
        self.screenshot("rules_example_4_3_basic_rules")
        self.rule_builder_sort("A", screenshot_name="rules_example_4_4_sort")

        self.rule_builder_add_regex_replacement("A", ".*", "http://www.uniprot.org/uniprot/\\0.fasta", screenshot_name="rules_example_4_5_url")
        self.screenshot("rules_example_4_6_url_built")
        self.rule_builder_set_mapping("list-identifiers", "A")
        self.rule_builder_set_mapping("url", "C")
        self.rule_builder_set_extension("fasta")
        self.rule_builder_set_collection_name("UP000052092")
        self.screenshot("rules_example_4_7_mapping_extension_and_name")

        rule_builder.view_source.wait_for_and_click()
        text_area_elem = rule_builder.source.wait_for_visible()

        self.screenshot("rules_example_4_8_source")
        self.write_screenshot_directory_file("rules_example_4_8_text", text_area_elem.get_attribute("value"))

        rule_builder.main_button_ok.wait_for_and_click()
        rule_builder.view_source.wait_for_visible()

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_5_matching_collections(self):
        self._setup_uniprot_example()
        self._wait_for_upload_modal()
        self.screenshot("rules_example_5_1_inputs")
        self.upload_rule_build()

        rule_builder = self.components.rule_builder
        rule_builder.view_source.wait_for_and_click()

        content = self._read_rules_test_data_file("uniprot.json")
        self.rule_builder_enter_source_text(content)
        self.screenshot("rules_example_5_2_source")
        rule_builder.main_button_ok.wait_for_and_click()
        rule_builder.view_source.wait_for_visible()
        self.screenshot("rules_example_5_3_initial_rules")
        self.rule_builder_add_value("fasta", screenshot_name="rules_example_5_4_fasta")
        self.rule_builder_add_value("UP000052092 FASTA")
        self.rule_builder_add_regex_replacement("A", ".*", "http://www.uniprot.org/uniprot/\\0.gff", screenshot_name="rules_example_5_5_url")
        self.rule_builder_add_value("gff3")
        self.rule_builder_add_value("UP000052092 GFF3")
        self._scroll_to_end_of_table()
        self.screenshot("rules_example_5_6_presplit")
        self.rule_builder_split_columns(["C", "D", "E"], ["F", "G", "H"], screenshot_name="rules_example_5_7_split_columns")
        self.screenshot("rules_example_5_8_split")
        self.rule_builder_set_mapping("file-type", "D")
        self.rule_builder_set_mapping("collection-name", "E")
        self.screenshot("rules_example_5_9_mapping")

    @selenium_test
    @pytest.mark.gtn_screenshot
    @pytest.mark.local
    def test_rules_example_6_nested_lists(self):
        self.home()
        self.perform_upload(self.get_filename("rules/PRJNA355367.tsv"))
        self.history_panel_wait_for_hid_ok(1)
        self.upload_rule_start()
        self.upload_rule_set_data_type("Collection")
        self.upload_rule_set_input_type("History Dataset")
        self.upload_rule_set_dataset(1)

        self._wait_for_upload_modal()
        self.screenshot("rules_example_6_1_paste")
        self.upload_rule_build()

        rule_builder = self.components.rule_builder
        rule_builder._.wait_for_and_click()

        self.screenshot("rules_example_6_2_rules_landing")

        self.rule_builder_filter_count(1)

        self.rule_builder_set_mapping("url", "J")
        self.rule_builder_set_extension("sra")
        self._scroll_to_end_of_table()
        self.screenshot("rules_example_6_3_end_of_table")
        self.rule_builder_add_regex_groups("L", 1, r"([^\d]+)\d+", screenshot_name="rules_example_6_4_regex")
        self.rule_builder_set_mapping("list-identifiers", ["M", "A"], screenshot_name="rules_example_6_5_multiple_identifiers_edit")
        self._scroll_to_end_of_table()
        self.screenshot("rules_example_6_6_multiple_identifiers")
        self.rule_builder_set_collection_name("PRJNA355367")
        self.screenshot("rules_example_6_7_named")

    def _read_rules_test_data_file(self, name):
        with open(self.test_data_resolver.get_filename(os.path.join("rules", name))) as f:
            return f.read()

    def _wait_for_upload_modal(self):
        self.components.upload.build_btn.wait_for_visible()
        self.components.upload.build_btn.wait_for_clickable()

    def _scroll_to_end_of_table(self):
        rule_builder = self.components.rule_builder
        table_elem = rule_builder.table.wait_for_visible()
        first_cell = table_elem.find_elements_by_css_selector("td")[0]
        action_chains = self.action_chains()
        action_chains.move_to_element(first_cell)
        action_chains.click(first_cell)
        for _ in range(15):
            action_chains.send_keys(Keys.ARROW_RIGHT)
        action_chains.perform()

    def _setup_uniprot_example(self):
        self.perform_upload(self.get_filename("rules/uniprot.tsv"))
        self.history_panel_wait_for_hid_ok(1)
        self.upload_rule_start()
        self.upload_rule_set_data_type("Collection")
        self.upload_rule_set_input_type("History Dataset")
        self.upload_rule_set_dataset(1)

    # @selenium_test
    # def test_rules_example_5_matching_collections(self):
    #    self.rule_builder_add_value("Protein FASTA")
    #     self.rule_builder_add_value("gff")
    #    self.rule_builder_add_value("Protein GFF")



from galaxy_test.base.populators import flakey
from .framework import (
    managed_history,
    selenium_test,
    SeleniumTestCase,
)

JUPYTER_VISUALIZATION_NAME = "Jupyter"


class JupyterTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    @flakey
    @managed_history
    def test_jupyter_launch(self):
        self._stage_test_data_and_launch()
        with self.main_panel():
            self.components.gies.spinner.wait_for_absent(wait_type=self.wait_types.GIE_SPAWN)
            viz_iframe = self.components.gies.iframe.wait_for_visible(wait_type=self.wait_types.GIE_SPAWN)
            try:
                self.driver.switch_to.frame(viz_iframe)
                self.components.gies.jupyter.body.wait_for_visible()
                trusted_element = self.components.gies.jupyter.trusted_notification.wait_for_visible()
                assert trusted_element.text == "Trusted"
            finally:
                self.driver.switch_to.default_content()

    @selenium_test
    @flakey
    @managed_history
    def test_jupyter_interaction(self):
        self._stage_test_data_and_launch()
        with self.main_panel():
            self.components.gies.spinner.wait_for_absent(wait_type=self.wait_types.GIE_SPAWN)
            viz_iframe = self.components.gies.iframe.wait_for_visible(wait_type=self.wait_types.GIE_SPAWN)
            try:
                self.driver.switch_to.frame(viz_iframe)
                self.components.gies.jupyter.body.wait_for_visible()
                self.wait_for_selector("ul.nav")
                li_links = self.driver.find_elements_by_css_selector("ul.nav li.dropdown a.dropdown-toggle")
                cell_li = None
                found_li_texts = []
                for li_link in li_links:
                    li_text = li_link.text
                    found_li_texts.append(li_text)
                    if li_link.text == "Cell":
                        cell_li = li_link
                        break
                if cell_li is None:
                    raise Exception(f"Failed to find 'Cell' drop down menu, found menu options {found_li_texts}")
                cell_li.click()
                self.wait_for_and_click_selector("ul#cell_menu li#run_all_cells")
            finally:
                self.driver.switch_to.default_content()

        self.sleep_for(wait_type=self.wait_types.JOB_COMPLETION)
        self.history_panel_refresh_click()

        # Re-running notebook should publish two new things to the history.
        self.history_panel_wait_for_hid_ok(3)
        self.history_panel_wait_for_hid_ok(4)

    def _stage_test_data_and_launch(self):
        self.perform_upload(self.get_filename("1.txt"))
        self.perform_upload(self.get_filename("selenium-test-notebook.ipynb"))
        self.wait_for_history()
        self.history_panel_ensure_showing_item_details(2)
        self.history_panel_item_click_visualization_menu(2)
        self.ensure_visualization_available(2, JUPYTER_VISUALIZATION_NAME)
        self.history_panel_item_click_visualization(2, JUPYTER_VISUALIZATION_NAME)
        # Wait for the link to update the content to main panel, without this sleep I think
        # sometimes Selenium is waiting in the previous page (presumably Galaxy's welcome).
        self.sleep_for(wait_type=self.wait_types.UX_TRANSITION)



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class HistoryExportTestCase(SeleniumTestCase):
    ensure_registered = True

    @selenium_test
    def test_history_export(self):
        self.perform_upload_of_pasted_content("my cool content")
        self.history_panel_wait_for_hid_ok(1)

        self.select_export_to_file()

        history_export = self.components.history_export
        history_export.export_link.wait_for_and_click()
        history_export.running.wait_for_visible()
        history_export.running.wait_for_absent(wait_type=self.wait_types.JOB_COMPLETION)
        history_export.generated_export_link.wait_for_visible()
        history_export.copy_export_link.wait_for_visible()
        history_export.job_table.assert_absent_or_hidden()
        history_export.show_job_link.wait_for_and_click()
        history_export.job_table.wait_for_present()
        history_export.job_table_ok.wait_for_and_click()
        history_export.job_table.wait_for_absent()

        self.select_export_to_file()

        # this time the exported link is still there
        history_export.generated_export_link.wait_for_visible()
        history_export.export_link.assert_absent()

        self.perform_upload_of_pasted_content("my cool content part 2")
        self.history_panel_wait_for_hid_ok(2)

        self.select_export_to_file()

        # now we have a generated link and a link to update to the newest export
        history_export.generated_export_link.wait_for_visible()
        history_export.export_link.wait_for_visible()

    def select_export_to_file(self):
        if self.is_beta_history():
            self.use_bootstrap_dropdown(option="export to file", menu="history menu")
        else:
            self.click_history_options()
            self.components.history_panel.options_show_export_history_to_file.wait_for_and_click()



from .framework import (
    managed_history,
    selenium_test,
    SeleniumTestCase
)


class HistoryStructureTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    @managed_history
    def test_history_structure(self):
        def assert_details(expected_to_be_visible):

            error_message = "details are visible!"
            if expected_to_be_visible:
                error_message = "details are not visible!"

            assert expected_to_be_visible == self.components.history_structure.details.is_displayed, error_message

        self.perform_upload(self.get_filename("1.fasta"))
        self.wait_for_history()
        self.components.history_structure.header.assert_absent_or_hidden()
        self.components.history_structure.dataset.assert_absent_or_hidden()

        self.history_panel_show_structure()

        # assert details are not visible before expand
        self.components.history_structure.details.assert_absent_or_hidden()

        self.components.history_structure.dataset.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)

        assert_details(True)
        self.components.history_structure.header.wait_for_and_click()
        self.sleep_for(self.wait_types.UX_RENDER)

        assert_details(False)
        self.components.history_structure.dataset.assert_absent_or_hidden()



from .framework import (
    selenium_test,
    SeleniumTestCase
)


class HistoryCopyElementsTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_copy_hdca(self):
        history_id = self.current_history_id()
        input_collection = self.dataset_collection_populator.create_list_in_history(history_id, contents=["0", "1", "0", "1"]).json()
        input_hid = input_collection["hid"]

        failed_response = self.dataset_populator.run_exit_code_from_file(history_id, input_collection["id"])
        failed_collection = failed_response["implicit_collections"][0]
        failed_hid = failed_collection["hid"]

        if not self.is_beta_history():
            self.home()

        self.history_panel_wait_for_hid_state(input_hid, "ok")
        self.history_panel_wait_for_hid_state(failed_hid, "error")
        self.history_panel_click_copy_elements()

        with self.main_panel():
            self.components.history_copy_elements.collection_checkbox(id=input_collection["id"]).wait_for_and_click()
            self.components.history_copy_elements.collection_checkbox(id=failed_collection["id"]).wait_for_and_click()

            text_element = self.components.history_copy_elements.new_history_name.wait_for_and_click()
            text_element.send_keys("newhistfor_copy_hdca")
            self.components.history_copy_elements.copy_button.wait_for_and_click()
            self.sleep_for(self.wait_types.UX_TRANSITION)
            self.components.history_copy_elements.done_link.wait_for_and_click()

        # I don't know why this sleep is necessary but it seems to be
        self.sleep_for(self.wait_types.UX_RENDER)
        # Okay copied first
        self.history_panel_wait_for_hid_state(5, "ok")
        # Then 4 datasets and then the failed collection (this was six when coming from the original history)
        self.history_panel_wait_for_hid_state(10, "error")



import time

import pytest

from galaxy.selenium.navigates_galaxy import (
    edit_details,
    WAIT_TYPES
)
from galaxy_test.base.api_asserts import assert_status_code_is
from galaxy_test.base.populators import flakey
from .framework import (
    retry_assertion_during_transitions,
    selenium_test,
    SeleniumTestCase
)


class HistoryPanelCollectionsTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_mapping_collection_states_terminal(self):
        history_id = self.current_history_id()
        input_collection, failed_collection = self._generate_partially_failed_collection_with_input()
        input_hid = input_collection["hid"]
        failed_hid = failed_collection["hid"]
        ok_inputs = {
            "input1": {'batch': True, 'values': [{"src": "hdca", "id": input_collection["id"]}]},
            "sleep_time": 0,
        }
        ok_response = self.dataset_populator.run_tool(
            "cat_data_and_sleep",
            ok_inputs,
            history_id,
        )
        ok_hid = ok_response["implicit_collections"][0]["hid"]

        if not self.is_beta_history():
            # sleep really shouldn't be needed :(
            time.sleep(1)

            self.home()

        self.history_panel_wait_for_hid_state(input_hid, "ok")
        self.history_panel_wait_for_hid_state(failed_hid, "error")
        self.history_panel_wait_for_hid_state(ok_hid, "ok")
        self.screenshot("history_panel_collections_state_mapping_terminal")

    @selenium_test
    @flakey  # Some times a Paste web thread will stall when jobs are running.
    def test_mapping_collection_states_running(self):
        history_id = self.current_history_id()
        input_collection = self.dataset_collection_populator.create_list_in_history(history_id, contents=["0", "1"]).json()
        running_inputs = {
            "input1": {'batch': True, 'values': [{"src": "hdca", "id": input_collection["id"]}]},
            "sleep_time": 60,
        }
        running_response = self.dataset_populator.run_tool_raw(
            "cat_data_and_sleep",
            running_inputs,
            history_id,
        )
        try:
            assert_status_code_is(running_response, 200)
            running_hid = running_response.json()["implicit_collections"][0]["hid"]

            # sleep really shouldn't be needed :(
            time.sleep(1)

            self.home()

            self.history_panel_wait_for_hid_state(running_hid, "running", 1)
            self.screenshot("history_panel_collections_state_mapping_running")
        finally:
            for job in running_response.json()["jobs"]:
                self.dataset_populator.cancel_job(job["id"])

    @selenium_test
    def test_output_collection_states_terminal(self):
        history_id = self.current_history_id()
        input_collection = self.dataset_collection_populator.create_list_in_history(history_id, contents=["0", "1", "0", "1"]).json()

        ok_inputs = {
            "input1": {"src": "hdca", "id": input_collection["id"]}
        }
        ok_response = self.dataset_populator.run_tool(
            "collection_creates_list",
            ok_inputs,
            history_id
        )
        ok_hid = ok_response["output_collections"][0]["hid"]
        assert ok_hid > 0

        failed_response = self.dataset_populator.run_tool(
            "collection_creates_dynamic_nested_fail",
            {},
            history_id,
        )
        failed_hid = failed_response["output_collections"][0]["hid"]
        assert failed_hid > 0

        # sleep really shouldn't be needed :(
        time.sleep(1)

        self.home()

        self.history_panel_wait_for_hid_state(ok_hid, "ok")
        self.history_panel_wait_for_hid_state(failed_hid, "error")

        self.screenshot("history_panel_collections_state_terminal")

    @selenium_test
    @flakey  # Some times a Paste web thread will stall when jobs are running.
    def test_output_collection_states_running(self):
        history_id = self.current_history_id()
        running_inputs = {
            "sleep_time": 180,
        }
        payload = self.dataset_populator.run_tool(
            "collection_creates_dynamic_nested",
            running_inputs,
            history_id,
        )
        assert payload["output_collections"]
        assert payload["jobs"]
        assert len(payload["jobs"]) > 0
        assert len(["output_collections"]) > 0

        running_hid = payload["output_collections"][0]["hid"]
        assert running_hid

        try:
            if not self.is_beta_history():
                # sleep really shouldn't be needed :(
                time.sleep(1)
                self.home()
            self.history_panel_wait_for_hid_state(running_hid, "running")
            self.screenshot("history_panel_collections_state_running")
        finally:
            for job in payload["jobs"]:
                self.dataset_populator.cancel_job(job["id"])

    @selenium_test
    def test_back_to_history_button(self):
        input_collection = self._populated_paired_and_wait_for_it()
        collection_hid = input_collection["hid"]
        self.history_panel_wait_for_hid_state(collection_hid, "ok")
        self._click_and_wait_for_collection_view(collection_hid)
        self._back_to_history()
        self.history_panel_wait_for_hid_state(collection_hid, "ok")

    def _back_to_history(self):
        if self.is_beta_history():
            # it's either a single back button or a dropdown with an option depending
            # on how deeply nested the collection is
            menu = self.beta_history_element("collection breadcrumbs menu")
            if not menu.is_absent:
                # menu.is_displayed results in an error but I don't understand why
                menu.wait_for_and_click()
            back = self.beta_history_element("back to history")
        else:
            back = self.components.history_panel.collection_view.back
        back.wait_for_and_click()
        self.sleep_for(WAIT_TYPES.UX_RENDER)

    @selenium_test
    def test_rename_collection(self):
        input_collection = self._populated_paired_and_wait_for_it()
        collection_hid = input_collection["hid"]
        self.history_panel_wait_for_hid_state(collection_hid, "ok")
        collection_view = self._click_and_wait_for_collection_view(collection_hid)

        new_name = "My New Name"

        if self.is_beta_history():
            self._rename_collection(new_name)
            self.screenshot("history_panel_collection_view_rename_beta")
            title_element = self.beta_history_element("collection name display").wait_for_present()
        else:
            title_element = collection_view.title.wait_for_visible()
            assert title_element.text == input_collection["name"]
            title_element.click()
            title_rename_element = collection_view.title_input.wait_for_visible()
            title_rename_element.send_keys(new_name)
            self.send_enter(title_rename_element)
            self.screenshot("history_panel_collection_view_rename")
            title_element = collection_view.title.wait_for_visible()
        assert title_element.text == new_name

    @edit_details
    def _rename_collection(self, new_name):
        title_element = self.beta_history_element("name input").wait_for_clickable()
        title_element.clear()
        title_element.send_keys(new_name)

    @selenium_test
    def test_name_tags_display(self):
        # Test setting a name tag and viewing it from the outer history panel.
        input_collection = self._populated_paired_and_wait_for_it()
        collection_hid = input_collection["hid"]
        self._click_and_wait_for_collection_view(collection_hid)

        if self.is_beta_history():
            self.history_panel_add_tags(["#moo"])
        else:
            # the space on the end of the parent_selector is important
            self.tagging_add(["#moo"], parent_selector=".dataset-collection-panel .controls ")

        self.sleep_for(WAIT_TYPES.HISTORY_POLL)
        self.screenshot("history_panel_collection_view_add_nametag")
        self._back_to_history()

        self.history_panel_wait_for_hid_state(collection_hid, "ok")
        nametags = self.history_panel_item_get_nametags(collection_hid)
        assert nametags == ["moo"]
        self.screenshot("history_panel_collection_with_nametag")

    @selenium_test
    def test_paired_display(self):
        input_collection = self._populated_paired_and_wait_for_it()
        collection_hid = input_collection["hid"]
        collection_view = self._click_and_wait_for_collection_view(collection_hid)

        if self.is_beta_history():
            self.sleep_for(WAIT_TYPES.HISTORY_POLL)
            dataset_elements = self.content_item_by_attributes().all()
        else:
            dataset_elements = collection_view.list_items.all()

        assert len(dataset_elements) == 2, dataset_elements
        titles = [de.find_element_by_css_selector(".title .name").text for de in dataset_elements]
        assert titles == ["forward", "reverse"]
        self.screenshot("history_panel_collection_view_paired")

    @selenium_test
    @flakey  # Fails only in Jenkins full suite - possibly due to #3782
    def test_list_display(self):
        input_collection, failed_collection = self._generate_partially_failed_collection_with_input()
        failed_hid = failed_collection["hid"]

        self.home()
        self.history_panel_wait_for_hid_state(failed_hid, "error")
        collection_view = self._click_and_wait_for_collection_view(failed_hid)

        @retry_assertion_during_transitions
        def check_four_datasets_shown():
            if self.is_beta_history():
                self.sleep_for(WAIT_TYPES.HISTORY_POLL)
                dataset_elements = collection_view.list_items_beta.all()
            else:
                dataset_elements = collection_view.list_items.all()
            assert len(dataset_elements) == 4, dataset_elements
            title_elements = [de.find_element_by_css_selector(".title .name").text for de in dataset_elements]
            assert title_elements == ["data1", "data2", "data3", "data4"]

        check_four_datasets_shown()
        self.screenshot("history_panel_collection_view_list")

    @selenium_test
    def test_list_paired_display(self):
        history_id = self.current_history_id()
        collection = self.dataset_collection_populator.create_list_of_pairs_in_history(history_id).json()["outputs"][0]
        collection_hid = collection["hid"]
        self.home()
        self.history_panel_wait_for_hid_state(collection_hid, "ok")
        self._click_and_wait_for_collection_view(collection_hid)
        self.screenshot("history_panel_collection_view_list_paired")

    @selenium_test
    def test_list_list_display(self):
        history_id = self.current_history_id()
        collection = self.dataset_collection_populator.create_list_of_list_in_history(history_id).json()
        collection_hid = collection["hid"]
        self.home()
        self.history_panel_wait_for_hid_state(collection_hid, "ok")
        self._click_and_wait_for_collection_view(collection_hid)
        self.screenshot("history_panel_collection_view_list_list")

    @selenium_test
    def test_limiting_collection_rendering(self):
        if self.is_beta_history():
            raise pytest.skip("Beta history can scroll through collections of any length")
        history_id = self.current_history_id()
        collection = self.dataset_collection_populator.create_list_in_history(history_id, contents=["0", "1", "0", "1"]).json()
        collection_hid = collection["hid"]

        with self.local_storage("collectionFuzzyCountDefault", 2):
            self.home()

            self.history_panel_wait_for_hid_state(collection_hid, "ok")
            self._click_and_wait_for_collection_view(collection_hid)
            self.screenshot("history_panel_collection_view_limiting")
            warning_text = self.components.history_panel.collection_view.elements_warning.wait_for_text()
            assert "only 2 of 4 items" in warning_text, warning_text

    def _generate_partially_failed_collection_with_input(self):
        history_id = self.current_history_id()
        input_collection = self.dataset_collection_populator.create_list_in_history(history_id, contents=["0", "1", "0", "1"]).json()
        failed_response = self.dataset_populator.run_exit_code_from_file(history_id, input_collection["id"])
        failed_collection = failed_response["implicit_collections"][0]
        return input_collection, failed_collection

    def _click_and_wait_for_collection_view(self, collection_hid):
        self.history_panel_click_item_title(collection_hid)

        collection_view = self.components.history_panel.collection_view
        collection_view._.wait_for_present()
        return collection_view

    def _populated_paired_and_wait_for_it(self):
        history_id = self.current_history_id()
        input_collection = self.dataset_collection_populator.create_pair_in_history(history_id).json()
        collection_hid = input_collection["hid"]
        if not self.is_beta_history():
            self.home()
        self.history_panel_wait_for_hid_state(collection_hid, "ok")

        return input_collection



from .framework import (
    managed_history,
    selenium_test,
    SeleniumTestCase,
)


class PagesTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    @managed_history
    def test_simple_page_creation_edit_and_view(self):
        # Upload a file to test embedded object stuff
        test_path = self.get_filename("1.fasta")
        self.perform_upload(test_path)
        self.history_panel_wait_for_hid_ok(1)
        self.navigate_to_pages()
        self.screenshot("pages_grid")

        self.components.pages.create.wait_for_and_click()
        name = self._get_random_name(prefix="page")
        slug = self._get_random_name(prefix="pageslug")
        self.tool_set_value("title", name)
        self.tool_set_value("slug", slug)
        self.tool_set_value("content_format", "HTML", expected_type="select")
        self.screenshot("pages_create_form")

        # Sometimes 'submit' button not yet hooked up?
        self.sleep_for(self.wait_types.UX_RENDER)

        self.components.pages.submit.wait_for_and_click()

        self.click_grid_popup_option(name, "Edit content")
        self.components.pages.editor.wym_iframe.wait_for_visible()
        self.screenshot("pages_editor_new")
        self.driver.switch_to.frame(0)
        try:
            self.components.pages.editor.wym_iframe_content.wait_for_and_send_keys("moo\n\n\ncow\n\n")
        finally:
            self.driver.switch_to.default_content()

        self.components.pages.editor.embed_button.wait_for_and_click()
        self.screenshot("pages_editor_embed_menu")
        self.components.pages.editor.embed_dataset.wait_for_and_click()
        saved_datasets_element = self.components.pages.editor.dataset_selector.wait_for_and_click()
        self.screenshot("pages_editor_embed_dataset_dialog")
        checkboxes = saved_datasets_element.find_elements_by_css_selector("input[type='checkbox']")
        assert len(checkboxes) > 0
        checkboxes[0].click()
        self.components.pages.editor.embed_dialog_add_button.wait_for_and_click()

        self.sleep_for(self.wait_types.UX_RENDER)
        self.components.pages.editor.save.wait_for_and_click()
        self.screenshot("pages_editor_saved")
        self.home()
        self.navigate_to_pages()
        self.click_grid_popup_option(name, "View")
        self.screenshot("pages_view_simple")

    def navigate_to_pages(self):
        self.click_masthead_user()  # Open masthead menu
        self.components.masthead.pages.wait_for_and_click()



from galaxy_test.base.workflow_fixtures import WORKFLOW_SIMPLE_CAT_TWICE
from .framework import (
    selenium_test,
    SeleniumTestCase,
)


class WorkflowSharingTestCase(SeleniumTestCase):

    ensure_registered = True

    @selenium_test
    def test_share_workflow_with_login_redirect(self):
        user_email = self.get_logged_in_user()["email"]
        workflow_id = self.workflow_populator.upload_yaml_workflow(WORKFLOW_SIMPLE_CAT_TWICE)
        self.logout()
        sharing_url = f"workflow/sharing?id={workflow_id}"
        self.get(sharing_url)
        modal = self.components._.messages.error.wait_for_present()
        assert modal.text == 'You must be logged in to Share or export Galaxy workflows.'
        login_link = modal.find_element_by_tag_name('a')
        assert login_link.text == 'logged in'
        self.sleep_for(self.wait_types.UX_RENDER)
        login_link.click()
        form = self.wait_for_visible(self.navigation.login.selectors.form)
        self.fill(form, {'login': user_email, 'password': self.default_password})
        self.wait_for_and_click(self.navigation.login.selectors.submit)
        self.wait_for_logged_in()
        accessible_via_link_button = self.wait_for_selector('.make-accessible')
        accessible_via_link_button.click()
        self.wait_for_xpath('//strong[text()="accessible via link"]')
